"""Docstring for Ranking.py module.

This module contains the differents ranking function to rank the library by task, by theme or globaly

"""

import numpy as np

from genbenchsite.src.task import Task
from genbenchsite.src.library import Library
from pprint import pprint


def RankingMethod(*args, **kwargs):
    return LexMaxWithThreshold(*args, **kwargs)


def RankingLibraryByTask(threshold=0.0, isResultList=True) -> dict[str, list[str]]:
    r"""Rank all the Library by their results for each task.

    Each library has a list of result for each task. For each result we apply the LexMax algorithm
    on the argument of the task. The result is a list of library name sorted by their rank compiled inside a
    dictionary with the task name as key. The threshold is used to remove the result with an argument that are under the threshold.

    Parameters
    ----------
    threshold : float, default=0.0
        The threshold to remove the result with an argument that are under the threshold.

    isResultList : bool, default=True
        If True, the result will be a list of library name sorted by their rank. If False, the result will be a dictionary with the library name as key and their rank as value.

    Returns
    -------
    dictionaryTaskLibraryResults : dict of str and list of str
        A dictionary with the task name as key and a list of library name sorted by their rank as value.

    See Also
    --------
    LexMax : The LexMax algorithm.
    LexMaxWithThreshold : The LexMax algorithm with a threshold.

    Examples
    --------
    >>> from task import Task
    >>> from library import Library
    >>> from json_to_python_object import FileReaderJson
    >>> libraryList, taskList = FileReaderJson("data.json")
    >>> print(RankingLibraryByTask())
    {'Task1': ['Library1', 'Library2', 'Library3'], 'Task2': ['Library1', 'Library2', 'Library3'], 'Task3': ['Library1', 'Library2', 'Library3']}
    """
    rank_over_all_aspect = {}
    # we collect the ranking by runtime and by eval
    rank_task_runtime = RankingLibraryByTaskRuntime(threshold, isResultList=False)
    rank_task_eval = RankingLibraryByTaskEval(threshold, isResultList=False)
    # we merge the two ranking to be able to lexmax them
    for taskName in Task.GetAllTaskName():
        dictionaryTaskLibraryResults = []
        rank_over_all_aspect[taskName] = {}
        # we only add the eval ranking if there is an eval function for the task
        dictionaryTaskLibraryResults.append(rank_task_eval[taskName]) if len(
            rank_task_eval[taskName]
        ) > 0 else None
        dictionaryTaskLibraryResults.append(rank_task_runtime[taskName])
        tmp = {}
        for library in Library.GetAllLibraryName():
            tmp[library] = [
                ranking[library]
                for ranking in dictionaryTaskLibraryResults
                if library in ranking.keys()
            ]

        rank_over_all_aspect[taskName] = RankingMethod(tmp, [], threshold=threshold)

    # we convert the result to a list if needed
    if isResultList:
        for taskName in rank_over_all_aspect.keys():
            rank_over_all_aspect[taskName] = list(rank_over_all_aspect[taskName].keys())

    return rank_over_all_aspect


def RankingLibraryByTaskRuntime(
    threshold=0.0, isResultList=True
) -> dict[str, list[str]]:
    r"""Rank all the Library by their results for each task.

    Each library has a list of result for each task. For each result we apply the LexMax algorithm
    on the argument of the task. The result is a list of library name sorted by their rank compiled inside a
    dictionary with the task name as key. The threshold is used to remove the result with an argument that are under the threshold.

    Parameters
    ----------
    threshold : float, default=0.0
        The threshold to remove the result with an argument that are under the threshold.

    Returns
    -------
    dictionaryTaskLibraryResults : dict of str and list of str
        A dictionary with the task name as key and a list of library name sorted by their rank as value.

    See Also
    --------
    LexMax : The LexMax algorithm.
    LexMaxWithThreshold : The LexMax algorithm with a threshold.

    Examples
    --------
    >>> from task import Task
    >>> from library import Library
    >>> from json_to_python_object import FileReaderJson
    >>> libraryList, taskList = FileReaderJson("data.json")
    >>> print(RankingLibraryByTask())
    {'Task1': ['Library1', 'Library2', 'Library3'], 'Task2': ['Library1', 'Library2', 'Library3'], 'Task3': ['Library1', 'Library2', 'Library3']}
    """
    # the goal here is to create a dictionary with the runtime of each library for each task
    dictionaryTaskLibraryRuntime = {}
    for taskName in Task.GetAllTaskName():
        dictionaryTaskLibraryRuntime[taskName] = {}
        for library in Library.GetLibraryByTaskName(taskName):
            dictionaryTaskLibraryRuntime[taskName][
                library.name
            ] = library.GetTaskByName(taskName).mean_runtime(library.name)

    # we apply the ranking method on the runtime
    for taskName in dictionaryTaskLibraryRuntime.keys():
        dictionaryTaskLibraryRuntime[taskName] = RankingMethod(
            dictionaryTaskLibraryRuntime[taskName],
            Task.GetTaskByName(taskName).arguments,
            threshold,
        )

    # we convert the result to a list if needed
    if isResultList:
        for taskName in dictionaryTaskLibraryRuntime.keys():
            dictionaryTaskLibraryRuntime[taskName] = list(
                dictionaryTaskLibraryRuntime[taskName].keys()
            )

    return dictionaryTaskLibraryRuntime


def RankingLibraryByTaskEval(threshold=0.0, isResultList=True) -> dict[str, list[str]]:
    rank_task_eval = {}
    for taskName in Task.GetAllTaskName():
        dictionaryTaskLibraryResults = []
        rank_task_eval[taskName] = {}
        eval_function = Task.GetTaskByName(taskName).evaluation_function_name
        # if there is no evaluation function for the task, we skip it
        if len(eval_function) == 0:
            continue
        # we apply the ranking method on each evaluation function according to the sort order ( asc or desc )
        # can be changed in the config file of the task
        for function_name, sort_order in zip(
            eval_function, Task.GetTaskByName(taskName).evaluation_sort_order
        ):
            dictionaryTaskLibraryResults.append(
                RankingLibraryByEvaluation(
                    taskName, function_name, threshold, sort_order
                )
            )

        # we merge the ranking of each evaluation function
        # so we have a dictionary with the library name as key and a list of rank for each evaluation function
        rank_task_eval[taskName] = RankingMethod(
            {
                library: [ranking[library] for ranking in dictionaryTaskLibraryResults]
                for library in Library.GetAllLibraryName()
            },
            [],
            threshold=threshold,
        )

    # we convert the result to a list if needed
    if isResultList:
        for taskName in rank_task_eval.keys():
            rank_task_eval[taskName] = list(rank_task_eval[taskName].keys())

    return rank_task_eval


def RankingLibraryByEvaluation(
    task_name, function_name, threshold=0, sort_order="desc"
) -> dict[str, list[str]]:
    """Rank all the Library by their results for each task.

    Each library has a list of result for each task. For each result we apply the LexMax algorithm
    on the argument of the task. The result is a list of library name sorted by their rank compiled inside a
    dictionary with the task name as key. The threshold is used to remove the result with an argument that are under the threshold.

    Parameters
    ----------
    task_name : str
        The name of the task.
    function_name : str
        The name of the evaluation function.
    threshold : float, default=0.0
        The threshold to remove the result with an argument that are under the threshold.

    Returns
    -------
    dictionaryTaskLibraryResults : dict of str and list of str
        A dictionary with the task name as key and a list of library name sorted by their rank as value.

    See Also
    --------
    LexMax : The LexMax algorithm.
    LexMaxWithThreshold : The LexMax algorithm with a threshold.

    Examples
    --------
    >>> from task import Task
    >>> from library import Library
    >>> from json_to_python_object import FileReaderJson
    >>> libraryList, taskList = FileReaderJson("data.json")


    """
    rank_task_function = {}
    # we check if the task exists and if it has an evaluation function
    if task_name not in Task.GetAllTaskName():
        return rank_task_function
    eval_function = Task.GetTaskByName(task_name).evaluation_function_name
    # if there is no evaluation function for the task, we skip it
    if len(eval_function) == 0 or function_name not in eval_function:
        return rank_task_function

    for library in Library.GetLibraryByTaskName(task_name):
        # we apply the ranking method on each evaluation function according to the sort order ( asc or desc )
        results = library.GetTaskByName(task_name).mean_evaluation(library.name)
        # if the task has not been evaluated, we set the rank to inf
        if all([r == float("inf") for r in results]):
            rank_task_function[library.name] = [float("inf") for r in results]
        else:
            rank_task_function[library.name] = [r.get(function_name) for r in results]
    rank_task_function = RankingMethod(
        rank_task_function,
        Task.GetTaskByName(task_name).arguments,
        threshold=threshold,
        reverse=sort_order == "asc",
    )
    return rank_task_function


def RankingLibraryByTheme(threshold=0, isResultList=True) -> dict[str, list[str]]:
    """Rank all the Library by their results for each theme.

    Each library has a list of result for each task. For each result we apply the LexMax algorithm
    on the argument of the task. The result is a list of library name sorted by their rank compiled inside a
    dictionary with the theme name as key. The threshold is used to remove the result with an argument that are under the threshold.

    Parameters
    ----------
    threshold : float, default=0.0
        The threshold to remove the result with an argument that are under the threshold.

    Returns
    -------
    dictionaryThemeLibraryResults : dict of str and list of str
        A dictionary with the theme name as key and a list of library name sorted by their rank as value.

    See Also
    --------
    LexMax : The LexMax algorithm.
    LexMaxWithThreshold : The LexMax algorithm with a threshold.

    Examples
    --------
    >>> from task import Task
    >>> from library import Library
    >>> from json_to_python_object import FileReaderJson
    >>> libraryList, taskList = FileReaderJson("data.json")
    >>> print(RankingLibraryByTheme())
    {'Theme1': ['Library1', 'Library2', 'Library3'], 'Theme2': ['Library1', 'Library2', 'Library3'], 'Theme3': ['Library1', 'Library2', 'Library3']}

    """
    rankLibraryByTask = RankingLibraryByTask(threshold=threshold, isResultList=False)
    rankLibraryByTheme = {}

    for theme in Task.GetAllThemeName():
        listTaskNameForCurrentTheme = Task.GetTaskNameByThemeName(theme)
        classementLibrary = {}
        for libraryName in Library.GetAllLibraryName():
            classementLibrary[libraryName] = [
                rankLibraryByTask[taskName][libraryName]
                for taskName in listTaskNameForCurrentTheme
            ]
        rankLibraryByTheme[theme] = LexMax(classementLibrary)

    if isResultList:
        for theme in rankLibraryByTheme.keys():
            rankLibraryByTheme[theme] = list(rankLibraryByTheme[theme].keys())

    return rankLibraryByTheme


def RankingLibraryGlobal(threshold=0, isResultList=True) -> list[str]:
    """Rank all the Library by their results for each theme.

    Each library has a list of result for each task. For each result we apply the LexMax algorithm
    on the argument of the task. The result is a list of library name sorted by their rank compiled inside a
    dictionary with the theme name as key. The threshold is used to remove the result with an argument that are under the threshold.

    Parameters
    ----------
    threshold : float, default=0.0
        The threshold to remove the result with an argument that are under the threshold.

    Returns
    -------
    list of str
        A list of library name sorted by their global rank.

    See Also
    --------
    LexMax : The LexMax algorithm.
    LexMaxWithThreshold : The LexMax algorithm with a threshold.

    """

    rankLibraryByTask = RankingLibraryByTask(threshold=threshold, isResultList=False)
    classementLibrary = {}
    for libraryName in Library.GetAllLibraryName():
        classementLibrary[libraryName] = []
        for taskName in rankLibraryByTask.keys():
            classementLibrary[libraryName].append(
                rankLibraryByTask[taskName][libraryName]
            )

    classementLibrary = LexMax(classementLibrary)

    if isResultList:
        classementLibrary = list(classementLibrary.keys())

    return classementLibrary


def LexMax(dictionnary: dict[str, list[float]], reverse=False) -> list[str]:
    r"""LexMax algorithm.

    The LexMax algorithm is used to rank dictionnary of result.

    Parameters
    ----------
    dictionnary : dict of str and list of float
        A dictionary with a value representing the all the results for a experiment.

    Returns
    -------
    list of str
        a list of the element sorted by their rank.

    Examples
    --------

    >>> dictionnary = {'Library1': [52.2, 42.1, 39.4], 'Library2': [45.2, 12.0, 80.2], 'Library3': [34.7, 15.8, 2.42]}
    >>> print(LexMax(dictionnary))
    ['Library3', 'Library2', 'Library1']

    # You can also use it on a dictionary with a list of rank
    >>> dictionnary = {'Library1': [3, 3, 2], 'Library2': [2, 1, 3], 'Library3': [1, 2, 1]}
    >>> print(LexMax(dictionnary))
    ['Library3', 'Library2', 'Library1']
    """
    rankMatrix = np.zeros((len(dictionnary.keys()), len(list(dictionnary.values())[0])))
    # On remplit la matrice avec les valeurs du dictionnaire
    for i, key in enumerate(dictionnary.keys()):
        for j, value in enumerate(dictionnary[key]):
            rankMatrix[i, j] = value

    # for each column we sort the value and we replace the value by their rank
    # the sort here will give a rank no matter the precision of the value
    for column in range(rankMatrix.shape[1]):
        rankMatrix[:, column] = [
            sorted(rankMatrix[:, column].tolist(), reverse=reverse).index(element)
            for element in rankMatrix[:, column].tolist()
        ]
    # print(rankMatrix)
    # we now sort the rank of each element to have a list of rank for each element sorted
    VectorLibrary = {}
    for i, key in enumerate(dictionnary.keys()):
        VectorLibrary[key] = sorted(rankMatrix[i, :].tolist())

    # print(VectorLibrary)
    # we can now compare the element by their list of rank
    sortedListRank = sorted(VectorLibrary.items(), key=lambda item: item[1])
    rk = 0
    elementRank = {}
    for i in range(len(sortedListRank)):
        elementRank[sortedListRank[i][0]] = rk
        # if the next element is the same, they share the same rank as the element are equivelent
        if (
            i < len(sortedListRank) - 1
            and sortedListRank[i][1] != sortedListRank[i + 1][1]
        ):
            rk += 1
    return elementRank


def LexMaxWithThreshold(
    dictionaryResults, argumentsList=list(), threshold=0, reverse=False
) -> list:
    """LexMax algorithm with a threshold.

    The LexMax algorithm is used to rank dictionnary of result. The threshold is used to remove the result with an argument that are strictly under the threshold.

    Parameters
    ----------
    dictionaryResults : dict of str and list of float
        A dictionary with a value representing the all the results for a experiment.
    argumentsList : list of float, default=list()
        A list of argument for each result.
    threshold : float, default=0.0
        The threshold to remove the result with an argument that are under the threshold.

    Returns
    -------
    list of str
        a list of the element sorted by their rank.

    See Also
    --------
    LexMax : The LexMax algorithm.

    Examples
    --------
    >>> dictionnary = {'Library1': [52.2, 42.1, 39.4], 'Library2': [45.2, 12.0, 80.2], 'Library3': [34.7, 15.8, 2.42]}
    >>> argumentsList = [0.1, 0.2, 0.3]
    >>> print(LexMaxWithThreshold(dictionnary, argumentsList, threshold=0.2))
    # Here only the result with an argument greater than 0.2 are used
    ['Library3', 'Library1', 'Library2']
    """
    if threshold == 0 or len(argumentsList) == 0:
        return LexMax(dictionaryResults, reverse=reverse)

    # On cherche la limite d'itération pour ne récuperer que les résultats dont
    # la valeur de l'argument est supérieur au seuil
    iterationLimit = 0
    for argument in argumentsList:
        if argument < threshold:
            iterationLimit += 1
        else:
            break

    # Si la limite d'itération est égale à la taille de la liste des arguments
    # cela veut dire que le seuil est trop élevé et que il n'y a pas de résultat
    if iterationLimit == len(argumentsList):
        # print("The threshold is too high, the LexMax algorithm will return without threshold")
        return LexMax(dictionaryResults, reverse=reverse)

    for key in dictionaryResults.keys():
        dictionaryResults[key] = dictionaryResults[key][iterationLimit:]

    return LexMax(dictionaryResults, reverse=reverse)


if __name__ == "__main__":
    from json_to_python_object import FileReaderJson

    _ = FileReaderJson(
        "results.json", "C:/Users/jules/Documents/Git/BenchSite/repository"
    )
    # _ = FileReaderJson(
    #     "results.json", "D:/Jules_Scolaire/Master_Androide_M1/BenchSite/repository"
    # )

    # print(f"RankingLibraryByTask : {RankingLibraryByTask(threshold=0)}")
    # print(f"RankingLibraryByTheme : {RankingLibraryByTheme(threshold=0)}")
    # print(f"RankingLibraryGlobal : {RankingLibraryGlobal(threshold=0)}")
    # pprint(f"RankingLibraryByTaskEval : {RankingLibraryByTaskEval(threshold=0)}")
    # pprint(f"RankingLibraryByTaskEval : {RankingLibraryByTaskRuntime(threshold=0,isResultList=False)}")
    pprint(f"RankingLibraryByTask : {RankingLibraryByTask(threshold=0)}")



import platform
import psutil
import multiprocessing
import datetime

import json


def GetRunMachineMetadata():
    """
    Get the metadata of the machine
    """
    return {
        "machine_os": platform.system(),
        "machine_os_version": platform.release(),
        "machine_os_architecture": platform.architecture(),
        "machine_processor": platform.processor(),
        "machine_processor_count": multiprocessing.cpu_count(),
        "machine_memory": psutil._common.bytes2human(psutil.virtual_memory().total),
        "machine_python_version": platform.python_version(),
        "execution_date": datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
    }


def SaveMachineDataInJson(outputFile: str):
    with open(outputFile, "w") as file:
        json.dump(GetRunMachineMetadata(), file)


if __name__ == "__main__":
    SaveMachineDataInJson("machine.json")



from configparser import ConfigParser
from pathlib import Path
from genbenchsite.src.logger import logger


class StructureTest:
    def __init__(self) -> None:
        pass

    def read_config(self, *pathConfig, listSection=[]):
        logger.info("Reading config file(s)")
        logger.debug(f"Path config : {pathConfig}")
        if len(pathConfig) == 0:
            logger.warning("No path given")
            return {}

        config = {}
        for path in pathConfig:
            logger.debug(f"Reading config file in {path.absolute()}")
            configParser = ConfigParser()
            configParser.read(path.absolute())
            sections = configParser.sections() if len(listSection) == 0 else listSection
            logger.debug(f"Sections : {sections}")
            refElement = path.parent.name
            logger.debug(f"Ref element : {refElement}")
            try:
                config[refElement] = {
                    key: configParser.get(section, key)
                    for section in sections
                    for key in configParser.options(section)
                }
            except Exception as e:
                logger.error(f"Error while reading config file : {e}")
                raise Exception(f"Error while reading config file : {e}")

        logger.info(
            "=======Config file(s) read=======\nnumber of section(s) found: "
            + str(len(config.keys()))
        )
        logger.debug(f"Config file : {config}")
        return config

    def find_config_file(self, path, name="config.ini"):
        path = Path(path)
        if not path.exists():
            logger.error(f"Path not found: {path}")
            raise FileNotFoundError(f"File not found: {path}")

        config_files = path.glob(f"**/{name}")
        if not config_files:
            logger.warning(f"Config file not found in {path}")
            raise FileNotFoundError(f"Config file not found in {path}")

        return config_files
    
    @classmethod
    def get_site_config(cls, pathSite):
        siteConfig = cls.read_config(
            *cls.find_config_file(pathSite),
            listSection=["site"],
        )
        return siteConfig["config"]
    
    @classmethod
    def get_target_config(cls, pathTarget):
        targetConfig = cls.read_config(
            *cls.find_config_file(pathTarget),
        )
        return targetConfig
    
    @classmethod
    def get_theme_config(cls, pathTheme):
        themeConfig = cls.read_config(
            *cls.find_config_file(pathTheme, name="theme.ini"),
            listSection=["theme"],
        )
        return themeConfig
    
    @classmethod
    def get_task_config(cls, pathTask):
        taskConfig = cls.read_config(
            *cls.find_config_file(pathTask),
            listSection=["task"],
        )
        return taskConfig
    
    @classmethod
    def get_benchmark_config(cls, pathBenchmark):
        benchmarkConfig = cls.read_config(
            *cls.find_config_file(Path(pathBenchmark) / "config"),
            listSection=["benchmark"],
        )
        return benchmarkConfig["config"]


if __name__ == "__main__":
    pathSite = "C:/Users/jules/Documents/Git/BenchSite/repository/config"
    listPathTarget = "C:/Users/jules/Documents/Git/BenchSite/repository/targets"
    themePath = "C:/Users/jules/Documents/Git/BenchSite/repository/themes"

    # test = StructureTest()
    # file_conf = test.findConfigFile(pathSite)
    # config = test.readConfig(*list(file_conf))

    # test = StructureTest()
    # file_conf = test.findConfigFile(listPathTarget)
    # config = test.readConfig(*list(file_conf))

    # test = StructureTest()
    # file_conf = test.find_config_file(themePath, "theme.ini")
    # config = test.read_config(*list(file_conf))

    print(StructureTest.get_site_config(pathSite))

    pass


from pathlib import Path
from genbenchsite.src.logger import logger


class Repository:
    def __init__(self, path):
        self.path = Path(path)
        self.file_list = []
        self.repository_list = []
        self.path.mkdir(parents=False, exist_ok=False)

    def __repr__(self):
        return (
            f"Repository({self.path})\n"
            + "\n".join([f"├── {f}" for f in self.file_list])
            + "\n"
            + "\n".join([f"└── {r}" for r in self.repository_list])
        )

    def add_file(self, file_name, content=""):
        self.file_list.append(file_name)
        with open(self.path / file_name, "w") as f:
            f.write(content)
        return self

    def add_repository(self, repository_name):
        new_repository = Repository(self.path / repository_name)
        self.repository_list.append(new_repository)
        return new_repository


def load_config(path):
    # check that the file exists
    if not Path(path).exists():
        raise ValueError(f"{path} does not exists")
    config = ""
    section = ["config", "task", "target", "theme",'readme']
    with open(path, "r") as f:
        config = f.read()
    return {s: c for s, c in zip(section, config.split("|"))}


def create_template(
    target_path="benchmark_name", nb_targets=2, nb_themes=2, nb_tasks=2
):
    # we look for the template file
    template_path = Path(__file__).parent / "template.txt"
    logger.info(f"Loading template from {template_path}")
    config = load_config(template_path)
    logger.info(f"Creating benchmark template in {target_path}")
    # we check that no repository already exists
    if Path(target_path).exists():
        logger.error(f"{target_path} already exists")
        raise ValueError(
            f"The benchmark or directory {target_path} already exists, please choose another name"
        )

    main_repo = Repository(target_path)
    main_repo.add_file("README.md", content=config["readme"])
    main_repo.add_repository("config").add_file("config.ini", content=config["config"])

    target_repo = main_repo.add_repository("targets")
    for i in range(nb_targets):
        target_repo.add_repository(f"target{i}").add_file(
            "config.ini", content=f"[target{i}]\n" + config["target"]
        )
    themes_repo = main_repo.add_repository("themes")
    for i in range(nb_themes):
        theme_repo = themes_repo.add_repository(f"theme{i}").add_file(
            "theme.ini", content=f"[theme{i}]\n" + config["theme"]
        )
        for j in range(nb_tasks):
            task_repo = (
                theme_repo.add_repository(f"task{j}")
                .add_file("config.ini", content=f"[task{i}]\n" + config["task"])
                .add_file(
                    "before_task.py",
                    content="# This file is used to prepare the task\ndef before_task(*args,**kwargs):\n    pass",
                )
                .add_file(
                    "evaluation_task.py",
                    content="# This file is executed to evaluate was has been done by the task\ndef evaluation_task(*args,**kwargs):\n    pass",
                )
                .add_file(
                    "extra_html_element.html",
                    content="<p>Here you can add a html element to the page to display</p>",
                )
            )
            for t in range(nb_targets):
                (
                    task_repo.add_file(
                        f"target{t}_before_run.py",
                        content="# This file is used to mesure the time before the task",
                    ).add_file(
                        f"target{t}_run.py",
                        content="# This file is executed to run the task (runtime = run - before_run)",
                    )
                )
            task_repo.add_repository("data")
    return main_repo


if __name__ == "__main__":
    print(create_template())



import logging
import time
from pathlib import Path

from rich.logging import RichHandler

logger = logging.getLogger(__name__)

logs_path = Path("logs")
logs_path.mkdir(exist_ok=True)

date_format = "%m_%d_%H_%M_%S"
max_logs = 10
file_name = f"debug_{time.strftime(date_format)}.log"

# we don't want the logs to over populate the folder
# so we delete the older logs

# we get all the dates of the logs
date_files = [file.name[6:-4] for file in logs_path.glob("*.log")]
# sort them
oldest_files_dates = sorted(date_files)[: -max_logs + 1]
# and get the names of the files to delete
oldest_files = [f"debug_{date}.log" for date in oldest_files_dates]
for file in oldest_files:
    (logs_path / file).unlink()

# logs_path = Path(__file__).parent
# file_name = "debug.log"

shell_file = logs_path / file_name

# the handler determines where the logs go: stdout/file
shell_handler = RichHandler()
file_handler = logging.FileHandler(shell_file, mode="w")

logger.setLevel(logging.DEBUG)
shell_handler.setLevel(logging.WARNING)
file_handler.setLevel(logging.DEBUG)

# the formatter determines what our logs will look like
# fmt_shell = "%(levelname)s %(asctime)s %(message)s" # no need level with rich
fmt_shell = "%(asctime)s %(message)s"
fmt_file = (
    "%(levelname)s %(asctime)s [%(filename)s:%(funcName)s:%(lineno)s] %(message)s"
)

shell_formatter = logging.Formatter(fmt_shell)
file_formatter = logging.Formatter(fmt_file)

# here we hook everything together
shell_handler.setFormatter(shell_formatter)
file_handler.setFormatter(file_formatter)

logger.addHandler(shell_handler)
logger.addHandler(file_handler)



from genbenchsite.src.static_site_generator import StaticSiteGenerator
from genbenchsite.src.structure_test import StructureTest
import os
from pathlib import Path

from genbenchsite.src.logger import logger
from genbenchsite.src.json_to_python_object import FileReaderJson
from genbenchsite.src.library import Library
from genbenchsite.src.task import Task
from genbenchsite.src.ranking import (
    RankingLibraryGlobal,
    RankingLibraryByTask,
    RankingLibraryByTheme,
)
from shutil import copyfile
from genbenchsite.src.collectCode import CollectCode
from genbenchsite.src.getMachineData import GetRunMachineMetadata

RemoveUnderscoreAndDash = lambda string: string.replace("_", " ").replace("-", " ")

ABOUT_URL = "https://white-on.github.io/BenchSite/"


class BenchSite:
    LEXMAX_THRESHOLD = 0
    DEFAULT_LOGO = "question.svg"
    DEFAULT_TASK_SCALE = "auto"
    DEFAULT_POST_TASK_SCALE = "auto"

    def __init__(
        self, inputFilename: str, outputPath="pages", structureTestPath="repository"
    ) -> None:
        logger.info("=======Creating BenchSite=======")

        FileReaderJson(inputFilename, structureTestPath)
        self.inputFilename = inputFilename
        self.outputPath = outputPath
        self.structureTestPath = structureTestPath

        logger.debug(f"inputFilename : {inputFilename}")
        logger.debug(f"outputPath : {outputPath}")
        logger.debug(f"structureTestPath : {structureTestPath}")

        # création du site statique
        # relative path to the script, assets and website folder
        self.staticSiteGenerator = StaticSiteGenerator(
            output_website_path=outputPath,
        )

        self.machineData = GetRunMachineMetadata()
        self.siteConfig = self.get_site_config()
        self.benchmarkConfig = self.GetBenchmarkConfig()
        self.setup_global_variables()

    def setup_global_variables(self):
        BenchSite.LEXMAX_THRESHOLD = int(
            self.siteConfig.get("threshold", BenchSite.LEXMAX_THRESHOLD)
        )
        BenchSite.DEFAULT_LOGO = self.siteConfig.get(
            "default_logo", BenchSite.DEFAULT_LOGO
        )
        BenchSite.DEFAULT_TASK_SCALE = self.siteConfig.get(
            "default_task_scale", BenchSite.DEFAULT_TASK_SCALE
        )

    def get_library_config(self):
        strtest = StructureTest()
        libraryConfig = strtest.read_config(
            *strtest.find_config_file(os.path.join(self.structureTestPath, "targets"))
        )
        return libraryConfig

    def get_task_config(self):
        strtest = StructureTest()
        taskConfig = strtest.read_config(
            *strtest.find_config_file(os.path.join(self.structureTestPath, "themes"))
        )
        return taskConfig

    def get_theme_config(self):
        strtest = StructureTest()
        themeConfig = strtest.read_config(
            *strtest.find_config_file(
                os.path.join(self.structureTestPath, "themes"), name="theme.ini"
            )
        )
        return themeConfig

    def get_site_config(self):
        strtest = StructureTest()
        siteConfig = strtest.read_config(
            *strtest.find_config_file(os.path.join(self.structureTestPath, "config")),
            listSection=["site"],
        )
        return siteConfig["config"]

    def GetBenchmarkConfig(self):
        """
        get the benchmark config from the config.ini file in the root folder
        """
        strTest = StructureTest()
        listBenchmarkpath = strTest.find_config_file(
            os.path.join(self.structureTestPath, "config"), name="config.ini"
        )
        benchmarkConfig = strTest.read_config(
            *listBenchmarkpath, listSection=["benchmark"]
        )
        return benchmarkConfig["config"]

    def GetLibraryLogo(self):
        logo = {}
        for libraryName in Library.GetAllLibraryName():
            # if the logo is present we copy it in the assets folder
            # we copy the logo in the assets folder
            if os.path.exists(
                os.path.join(self.structureTestPath, "targets", libraryName, "logo.png")
            ):
                copyfile(
                    os.path.join(
                        self.structureTestPath, "targets", libraryName, "logo.png"
                    ),
                    os.path.join(
                        self.outputPath,
                        self.staticSiteGenerator.assetsFilePath,
                        libraryName + ".png",
                    ),
                )
                logo[libraryName] = os.path.join(
                    self.staticSiteGenerator.assetsFilePath, libraryName + ".png"
                )

            else:
                # logo[libraryName] = os.path.join(self.staticSiteGenerator.assetsFilePath,"default.png")
                logo[libraryName] = os.path.join(
                    self.staticSiteGenerator.assetsFilePath, BenchSite.DEFAULT_LOGO
                )

        return logo

    def GenerateHTMLBestLibraryGlobal(self):
        contentfilePath = (
            os.path.basename(self.staticSiteGenerator.contentFilePath) + "/"
        )
        HTMLGlobalRanking = "<div id='global-rank' class='card'>\
                                <h1>Libraries</h1>\
                                <p>Current order for all libraries with all tasks take into account</p>\
                            <div class='grid'>"
        RankGlobal = RankingLibraryGlobal(
            threshold=BenchSite.LEXMAX_THRESHOLD, isResultList=False
        )
        HTMLGlobalRanking += "".join(
            # [f"<div class='global-card'><p>{BenchSite.RankSubTitle(rank+1)} : {BenchSite.MakeLink(library)}</p></div>" for rank, library in enumerate(RankingLibraryGlobal(threshold=BenchSite.LEXMAX_THRESHOLD))])
            [
                f'<div class="global-card rank-arrow" data-rank="{RankGlobal}">{library}</div>'
                # f"<div class='global-card'><p>{BenchSite.MakeLink(contentfilePath + library,library)}</p></div>"
                for library in RankGlobal
            ]
        )
        HTMLGlobalRanking += "</div>\
                            </div>"
        return HTMLGlobalRanking

    @staticmethod
    def GenerateHTMLRankingAllTheme():
        HTMLThemeRanking = "<div id='theme-rank'>\
            <h1>Theme Ranking</h1>\
            <p>The best libraries for each theme</p>\
                <div class=\"grid\">"
        rankLibraryInTheme = RankingLibraryByTheme(threshold=BenchSite.LEXMAX_THRESHOLD)
        # On trie le dictionnaire par nom de thème pour avoir un classement par ordre alphabétique
        rankLibraryInTheme = {
            k: v
            for k, v in sorted(rankLibraryInTheme.items(), key=lambda item: item[0])
        }
        for theme in rankLibraryInTheme.keys():
            HTMLThemeRanking += f"<div class=\"card theme\"><h2>{theme}</h2><h3>{' '.join(taskName for taskName in Task.GetTaskNameByThemeName(theme))}</h3><ol>"
            HTMLThemeRanking += "".join(
                [f"<li>{library}</li>" for library in rankLibraryInTheme[theme]]
            )
            HTMLThemeRanking += "</ol></div>"
        HTMLThemeRanking += "</div></div>"
        return HTMLThemeRanking

    @staticmethod
    def GenerateHTMLRankingPerThemeName(themeName):
        HTMLThemeRanking = ""
        rankLibraryByTheme = RankingLibraryByTheme(threshold=BenchSite.LEXMAX_THRESHOLD)
        # HTMLThemeRanking += f"<div class=\"theme\"><h2>{themeName}</h2><h3>{' '.join(BenchSite.MakeLink(taskName) for taskName in Task.GetTaskNameByThemeName(themeName))}</h3>"
        HTMLThemeRanking += "<div class='grid'>" + "".join(
            # [f"<div class='card'><p>{BenchSite.RankSubTitle(rank)} : {BenchSite.MakeLink(library)}</div>" for rank, library in enumerate(rankLibraryByTheme[themeName])])
            [
                f"<div class='card'><p>{BenchSite.MakeLink(library)}</div>"
                for rank, library in enumerate(rankLibraryByTheme[themeName])
            ]
        )
        HTMLThemeRanking += "</div>"
        return HTMLThemeRanking

    def GenerateHTMLBestLibraryByTheme(self):
        contentfilePath = (
            os.path.basename(self.staticSiteGenerator.contentFilePath) + "/"
        )
        HTMLBestTheme = "<div id='theme-rank' class='card'>\
            <h1>Per Theme</h1>\
            <p>The best libraries for each theme</p>\
                <div class=\"grid\">"
        rankLibraryInTheme = RankingLibraryByTheme(
            threshold=BenchSite.LEXMAX_THRESHOLD,
            isResultList=False,
        )
        # # On trie le dictionnaire par nom de thème pour avoir un classement par ordre alphabétique
        # rankLibraryInTheme = {
        #     k: v
        #     for k, v in sorted(rankLibraryInTheme.items(), key=lambda item: item[0])
        # }
        for themeName in rankLibraryInTheme.keys():
            HTMLBestTheme += f"<div class='theme-card'><h2>{BenchSite.MakeLink(contentfilePath + themeName,themeName)}</h2><p>({', '.join(BenchSite.MakeLink(contentfilePath + taskName , taskName) for taskName in Task.GetTaskNameByThemeName(themeName))})</p>"
            HTMLBestTheme += f'<p class="rankBar" data-bar="{rankLibraryInTheme[themeName]}"></p></div>'
        HTMLBestTheme += "</div></div>"
        return HTMLBestTheme

    def GenerateHTMLMachineInfo(self):
        HTMLMachineInfo = (
            "<div class ='card' id='machine_info'><h1>Machine Informations</h1>"
        )
        machineData = self.machineData
        if machineData is None:
            HTMLMachineInfo += "<p>No machine informations available</p>"
        else:
            HTMLMachineInfo += "<ul>"
            for key in machineData.keys():
                HTMLMachineInfo += f"<li>{' '.join(key.split('_')[1:]).upper()} : <b>{machineData[key]}</b></li>"
            HTMLMachineInfo += "</ul>"
        HTMLMachineInfo += "</div>"
        return HTMLMachineInfo

    def GenerateHTMLBestLibraryByTask(self):
        contentfilePath = (
            os.path.basename(self.staticSiteGenerator.contentFilePath) + "/"
        )
        HTMLTask = (
            "<div id='task-rank' class='card'><h1>Per Task</h1><div class=\"grid\">"
        )
        rankLibraryInTask = RankingLibraryByTask(
            threshold=BenchSite.LEXMAX_THRESHOLD,
            isResultList=False,
        )
        for taskName in rankLibraryInTask.keys():
            HTMLTask += f'<div class="task-card rankBar" data-bar="{rankLibraryInTask[taskName]}"><h2>{BenchSite.MakeLink(contentfilePath + taskName, taskName)}</h2></div>'
            # HTMLTask += f"<div class='task-card'><h2>{BenchSite.MakeLink(contentfilePath + taskName, taskName)}</h2><p>{BenchSite.MakeLink(contentfilePath + highLightedLibrary, highLightedLibrary)}<p></div>"
        HTMLTask += "</div>"
        return HTMLTask

    @staticmethod
    def MakeLink(nameElement: str, strElement=None, a_balise_id=None) -> str:
        strElement = nameElement if strElement is None else strElement
        a_balise_id = f"id='{a_balise_id}'" if a_balise_id is not None else ""
        return f"<a href='{nameElement}.html' {a_balise_id}>{RemoveUnderscoreAndDash(strElement)}</a>"

    @staticmethod
    def RankSubTitle(rank: float) -> str:
        rank = int(rank)
        subtitle = ["st", "nd", "rd", "th"]
        return f"{rank}{subtitle[rank-1] if rank < 4 else subtitle[3]}"

    @staticmethod
    def OrderedList(listElement: list) -> str:
        return "&gt;".join([f"{element}" for element in listElement])

    @staticmethod
    def CreateScriptBalise(content="", scriptName=None, module: bool = False) -> str:
        moduleElement = "type='module'" if module else ""
        scriptFilePath = f"src ='{scriptName}'" if scriptName else ""
        return f"<script defer {moduleElement} {scriptFilePath}>{content}</script>"

    def GenerateStaticSite(self):
        staticSiteGenerator = self.staticSiteGenerator

        # ==================================================
        # HOME PAGE
        # ==================================================
        styleFilePath = "indexStyle.css"
        scriptFilePath = ""
        contentFilePath = os.path.basename(staticSiteGenerator.contentFilePath) + "/"
        linkTo = {
            "home": "index.html",
            # "about": f"{contentFilePath}about.html",
            "about": ABOUT_URL,
            "download": "results.json",
        }

        libraryConfig = self.get_library_config()
        taskConfig = self.get_task_config()
        themeConfig = self.get_theme_config()
        logoLibrary = self.GetLibraryLogo()

        logger.info("Generate HTML Home Page")
        logger.debug(f"library config : {libraryConfig}")
        logger.debug(f"task config : {taskConfig}")
        logger.debug(f"logo library : {logoLibrary}")

        social_media = list(
            map(
                lambda x: tuple(x.split(",")),
                self.siteConfig.get("social_media", {}).split(" "),
            )
        )
        codeLibrary = CollectCode(pathToInfrastructure=self.structureTestPath)

        # GOOGLEANALYTICS
        HTMLGoogleAnalytics = staticSiteGenerator.CreateHTMLComponent(
            "googleAnalytics.html",
            googleAnalyticsID=self.siteConfig.get("googleAnalyticsID", ""),
        )

        # HEADER
        HTMLHeader = staticSiteGenerator.CreateHTMLComponent(
            "header.html",
            styleFilePath=f"{staticSiteGenerator.styleFilePath}/{styleFilePath}",
            assetsFilePath=f"{staticSiteGenerator.assetsFilePath}",
            linkTo=linkTo,
            siteName=self.siteConfig.get("name", "No name attributed"),
            socialMediaList=social_media,
        )
        # NAVIGATION
        HTMLNavigation = staticSiteGenerator.CreateHTMLComponent(
            "navigation.html",
            TaskClassifiedByTheme={
                BenchSite.MakeLink(contentFilePath + theme, theme, f"{theme}-nav"): [
                    BenchSite.MakeLink(
                        contentFilePath + taskName, taskName, f"{taskName}-nav"
                    )
                    for taskName in Task.GetTaskNameByThemeName(theme)
                ]
                for theme in Task.GetAllThemeName()
            },
            librarylist=[
                "<li class='menu-item'>"
                + BenchSite.MakeLink(
                    contentFilePath + libraryName,
                    strElement=f"<img src='{logoLibrary[libraryName]}' alt='{libraryName}' class='logo'>{libraryName}",
                    a_balise_id=f"{libraryName}-nav",
                )
                + "</li>"
                for libraryName in Library.GetAllLibraryName()
            ],
            assetsFilePath=f"{staticSiteGenerator.assetsFilePath}",
        )

        # RANKING BAR GLOBALE
        HTMLGlobalRankingBar = staticSiteGenerator.CreateHTMLComponent(
            "rankBar.html",
            contentFolderPath=contentFilePath,
            dataGenerationDate=self.machineData["execution_date"],
            data=f"{RankingLibraryGlobal(threshold=BenchSite.LEXMAX_THRESHOLD,isResultList = False)}",
            scriptFilePath=f"./{staticSiteGenerator.scriptFilePath}/",
        )

        # PRESENTATION DE L'OUTIL
        HTMLPresentation = staticSiteGenerator.CreateHTMLComponent(
            "presentation.html",
            siteName=self.siteConfig["name"],
            siteDescription=self.siteConfig["description"],
        )

        # INFORMATIONS SUR LA MACHINE
        HTMLMachineInfo = self.GenerateHTMLMachineInfo()

        # CLASSEMENT GLOBAL
        HTMLGlobalRanking = self.GenerateHTMLBestLibraryGlobal()

        # CLASSEMENT DES MEILLEURS LIBRAIRIES PAR THEME
        HTMLThemeRanking = self.GenerateHTMLBestLibraryByTheme()

        # CLASSEMENT DES LIBRAIRIES PAR TACHES
        HTMLTaskRanking = self.GenerateHTMLBestLibraryByTask()

        HTMLMainContainer = (
            "<div id='main-container'>"
            + "".join(
                [
                    HTMLPresentation,
                    HTMLMachineInfo,
                    HTMLGlobalRanking,
                    HTMLThemeRanking,
                    HTMLTaskRanking,
                ]
            )
            + "</div>"
        )

        # FOOTER
        HTMLFooter = staticSiteGenerator.CreateHTMLComponent("footer.html")

        staticSiteGenerator.CreateHTMLPage(
            [
                HTMLHeader,
                HTMLNavigation,
                HTMLGlobalRankingBar,
                HTMLMainContainer,
                HTMLGoogleAnalytics,
                HTMLFooter,
            ],
            "index.html",
            manualOutputPath=os.path.split(staticSiteGenerator.contentFilePath)[0],
        )
        # ==================================================
        # TACHES PAGES
        # ==================================================

        styleFilePath = "taskStyle.css"
        scriptFilePath = "taskScript.js"
        linkTo = {
            "home": "../index.html",
            "about": ABOUT_URL,
            "download": "../results.json",
        }
        contentFilePath = "./"

        # NAVIGATION
        HTMLNavigation = staticSiteGenerator.CreateHTMLComponent(
            "navigation.html",
            TaskClassifiedByTheme={
                BenchSite.MakeLink(theme, theme, f"{theme}-nav"): [
                    BenchSite.MakeLink(
                        taskName, taskName, a_balise_id=f"{taskName}-nav"
                    )
                    for taskName in Task.GetTaskNameByThemeName(theme)
                ]
                for theme in Task.GetAllThemeName()
            },
            librarylist=[
                "<li class='menu-item'>"
                + BenchSite.MakeLink(
                    libraryName,
                    strElement=f"<img src='../{logoLibrary[libraryName]}' alt='{libraryName}' class='logo'>{libraryName}",
                    a_balise_id=f"{libraryName}-nav",
                )
                + "</li>"
                for libraryName in Library.GetAllLibraryName()
            ],
            assetsFilePath=f"../{staticSiteGenerator.assetsFilePath}",
        )
        # HEADER
        HTMLHeader = staticSiteGenerator.CreateHTMLComponent(
            "header.html",
            styleFilePath=f"../{staticSiteGenerator.styleFilePath}/{styleFilePath}",
            assetsFilePath=f"../{staticSiteGenerator.assetsFilePath}",
            linkTo=linkTo,
            siteName=self.siteConfig.get("name", "No name attributed"),
            socialMediaList=social_media,
        )

        taskRankDico = RankingLibraryByTask(
            threshold=BenchSite.LEXMAX_THRESHOLD, isResultList=False
        )

        for taskName in Task.GetAllTaskName():
            HTMLTaskRankingBar = staticSiteGenerator.CreateHTMLComponent(
                "rankBar.html",
                data=f"{taskRankDico[taskName]}",
                dataGenerationDate=self.machineData["execution_date"],
                scriptFilePath=f"../{staticSiteGenerator.scriptFilePath}/",
            )

            # CLASSEMENT DES LIBRAIRIES PAR TACHES

            # importedData = [task for task in Task.GetAllTaskByName(taskName)]
            # importedData = [[{"arg":r, "res":c} for r,c in zip(task.arguments,task.results)] for task in Task.GetAllTaskByName(taskName)]
            task = Task.GetTaskByName(taskName)
            importedRuntime = sum(
                [
                    [
                        {
                            "arguments": int(arg) if arg.isnumeric() else arg,
                            "runTime": runtime
                            if runtime != float("inf") and runtime > 0
                            else "error",
                            "libraryName": library.name,
                            "std": std if std != float("inf") and std > 0 else "error",
                        }
                        for arg, runtime, std in zip(
                            task.arguments_label,
                            task.mean_runtime(library.name),
                            task.standard_deviation_runtime(library.name),
                        )
                        # if runtime != float("inf")
                    ]
                    for library in Library.GetAllLibrary()
                ],
                [],
            )

            logger.debug(f"{importedRuntime = }")

            functionEvaluation = taskConfig[taskName].get("evaluation_function", None)
            if functionEvaluation is not None:
                functionEvaluation = functionEvaluation.split(" ")
            else:
                functionEvaluation = []

            task = Task.GetTaskByName(taskName)
            importedEvaluation = {
                function: sum(
                    [
                        [
                            {
                                "arguments": int(arg) if arg.isnumeric() else arg,
                                "runTime": res.get(function, 0)
                                if res.get(function, 0) != float("inf")
                                else "error",
                                "libraryName": library.name,
                                "std": std.get(function, 0)
                                if res.get(function, 0) != float("inf")
                                else "error",
                            }
                            for arg, res, std in zip(
                                task.arguments_label,
                                task.mean_evaluation(library.name),
                                task.standard_deviation_evaluation(library.name),
                            )
                            # if res.get(function) != float("inf")
                        ]
                        for library in Library.GetAllLibrary()
                    ],
                    [],
                )
                for function in functionEvaluation
            }

            logger.debug(f"{importedEvaluation = }")

            chartData = {}
            chartData["runtime"] = {
                "data": importedRuntime,
                "display": taskConfig[taskName].get("task_display", "groupedBar"),
                "label": "Runtime",
                "title": taskConfig[taskName].get("task_title", "Title"),
                "XLabel": taskConfig[taskName].get("task_xlabel", "X-axis"),
                "YLabel": taskConfig[taskName].get("task_ylabel", "Y-axis"),
                "scale": taskConfig[taskName].get("task_scale", "auto"),
                "timeout": taskConfig[taskName].get("timeout"),
            }
            for i, function in enumerate(functionEvaluation):
                xlabel = taskConfig[taskName].get("post_task_xlabel", "X-axis")
                ylabel = (
                    taskConfig[taskName].get("post_task_ylabel", "Y-axis").split(" ")
                )
                scale = taskConfig[taskName].get("post_task_scale", "auto").split(" ")
                title = taskConfig[taskName].get("post_task_title", "Title").split(",")
                chartData[function] = {
                    "data": importedEvaluation[function],
                    "display": taskConfig[taskName].get(
                        "post_task_display", "groupedBar"
                    ),
                    "label": function.capitalize(),
                    "title": title[i] if i < len(title) else title[0],
                    "XLabel": xlabel,
                    "YLabel": ylabel[i] if i < len(ylabel) else ylabel[0],
                    "scale": scale[i] if i < len(scale) else scale[0],
                }
            complementary_description = taskConfig[taskName].get(
                "extra_description", ""
            )
            # we're also adding information relevant to the task in the description
            complementary_description += "<br><br>"
            complementary_description += f"<p>Timeout : {taskConfig[taskName].get('timeout',self.benchmarkConfig.get('default_timeout','No timeout configured'))} (seconds)</p>"
            complementary_description += f"<p>Number of iteration : {taskConfig[taskName].get('nb_runs',self.benchmarkConfig.get('default_nb_runs','No number of iteration configured'))}</p>"
            complementary_description += f"<p>The task is interrupted if the number of timeout reached {self.benchmarkConfig.get('default_stop_after_x_timeout','No number of timeout configured')}</p>"

            HTMLExtra = taskConfig[taskName].get("extra_html_element", None)
            if HTMLExtra is not None:
                try:
                    HTMLExtra = list(
                        Path(self.structureTestPath).glob(f"**/{HTMLExtra}")
                    )[0].read_text()
                # if there is a typo in the extra html element
                except IndexError:
                    HTMLExtra = ""
                    logger.warning(
                        f"Extra HTML element {HTMLExtra} not found in the repository"
                    )
            else:
                HTMLExtra = ""

            # print(importedResults)
            # create the template for the code
            templateTask = ""
            for library in Library.GetAllLibrary():
                templateTask += f" <code id='{library.name}'>"
                templateTask += f" <h2>{library.name}</h2>"
                templateTask += f" {codeLibrary.get_code_HTML(library.name, taskName)}"
                templateTask += f" </code>"

            HTMLTaskRanking = staticSiteGenerator.CreateHTMLComponent(
                "task.html",
                taskName=RemoveUnderscoreAndDash(taskName),
                taskNamePage=BenchSite.CreateScriptBalise(
                    content=f"const TaskName = '{taskName}';"
                ),
                scriptFilePath=BenchSite.CreateScriptBalise(
                    scriptName=f"../{staticSiteGenerator.scriptFilePath}/{scriptFilePath}",
                    module=True,
                ),
                libraryOrdered=BenchSite.OrderedList(
                    RankingLibraryByTask(threshold=BenchSite.LEXMAX_THRESHOLD)[taskName]
                ),
                scriptData=BenchSite.CreateScriptBalise(
                    content=f"const importedData = {chartData};"
                ),
                code=templateTask,
                taskDescritpion=taskConfig[taskName].get(
                    "description", "No description"
                ),
                argumentsDescription=BenchSite.CreateScriptBalise(
                    content=f"const argDescription = '{taskConfig[taskName].get('arguments_description', 'No description')}';"
                ),
                displayScale=BenchSite.CreateScriptBalise(
                    content=f"const displayScale = '{taskConfig[taskName].get('display_scale', 'linear')}';"
                ),
                extra_html_element=HTMLExtra,
                extra_description=complementary_description,
            )

            staticSiteGenerator.CreateHTMLPage(
                [
                    HTMLHeader,
                    HTMLNavigation,
                    HTMLTaskRankingBar,
                    HTMLTaskRanking,
                    HTMLGoogleAnalytics,
                    HTMLFooter,
                ],
                f"{taskName}.html",
            )

        # ==================================================
        # THEME PAGES
        # ==================================================

        styleFilePath = "themeStyle.css"
        scriptFilePath = "themeScript.js"

        # HEADER
        HTMLHeader = staticSiteGenerator.CreateHTMLComponent(
            "header.html",
            styleFilePath=f"../{staticSiteGenerator.styleFilePath}/{styleFilePath}",
            assetsFilePath=f"../{staticSiteGenerator.assetsFilePath}",
            linkTo=linkTo,
            siteName=self.siteConfig.get("name", "No name attributed"),
            socialMediaList=social_media,
        )

        themeRankDico = RankingLibraryByTheme(
            threshold=BenchSite.LEXMAX_THRESHOLD, isResultList=False
        )

        for themeName in Task.GetAllThemeName():
            # CLASSEMENT DES LIBRAIRIES PAR TACHES BAR
            HTMLThemeRankingBar = staticSiteGenerator.CreateHTMLComponent(
                "rankBar.html",
                data=f"{themeRankDico[themeName]}",
                dataGenerationDate=self.machineData["execution_date"],
                scriptFilePath=f"../{staticSiteGenerator.scriptFilePath}/",
            )

            importedRuntime = sum(
                [
                    [
                        {
                            "taskName": taskName,
                            "libraryName": t,
                            "results": RankingLibraryByTask(
                                threshold=BenchSite.LEXMAX_THRESHOLD, isResultList=False
                            )[taskName][t],
                        }
                        for t in RankingLibraryByTask(
                            threshold=BenchSite.LEXMAX_THRESHOLD
                        )[taskName]
                    ]
                    for taskName in Task.GetTaskNameByThemeName(themeName)
                ],
                [],
            )
            summaryData = [
                {
                    # we need to consider the theme name as a task name
                    "taskName": themeName,
                    "libraryName": libraryName,
                    "results": themeRankDico[themeName][libraryName],
                }
                for libraryName in themeRankDico[themeName].keys()
            ]
            importedRuntime = summaryData + importedRuntime

            # CLASSEMENT DES LIBRAIRIES PAR TACHES
            HTMLThemeRanking = staticSiteGenerator.CreateHTMLComponent(
                "theme.html",
                themeName=RemoveUnderscoreAndDash(themeName),
                themeNamePage=BenchSite.CreateScriptBalise(
                    content=f"const themeName = '{themeName}';"
                ),
                taskNameList=", ".join(
                    BenchSite.MakeLink(taskName)
                    for taskName in Task.GetTaskNameByThemeName(themeName)
                ),
                results=self.GenerateHTMLRankingPerThemeName(themeName),
                scriptFilePath=BenchSite.CreateScriptBalise(
                    scriptName=f"../{staticSiteGenerator.scriptFilePath}/{scriptFilePath}",
                    module=True,
                ),
                scriptData=BenchSite.CreateScriptBalise(
                    content=f"const importedData = {importedRuntime};"
                ),
                description=themeConfig.get(themeName, {}).get(
                    "description", "No description attributed"
                ),
            )

            staticSiteGenerator.CreateHTMLPage(
                [
                    HTMLHeader,
                    HTMLNavigation,
                    HTMLThemeRankingBar,
                    HTMLThemeRanking,
                    HTMLGoogleAnalytics,
                    HTMLFooter,
                ],
                f"{themeName}.html",
            )

        # ==================================================
        # LIBRAIRIES PAGES
        # ==================================================

        styleFilePath = "libraryStyle.css"
        scriptFilePath = "libraryScript.js"

        libraryDico = RankingLibraryGlobal(
            threshold=BenchSite.LEXMAX_THRESHOLD, isResultList=False
        )
        # RANKING BAR GLOBALE
        HTMLGlobalRankingBar = staticSiteGenerator.CreateHTMLComponent(
            "rankBar.html",
            contentFolderPath=contentFilePath,
            data=f"{libraryDico}",
            dataGenerationDate=self.machineData["execution_date"],
            scriptFilePath=f"../{staticSiteGenerator.scriptFilePath}/",
        )

        for libraryName in Library.GetAllLibraryName():
            # HEADER
            HTMLHeader = staticSiteGenerator.CreateHTMLComponent(
                "header.html",
                styleFilePath=f"../{staticSiteGenerator.styleFilePath}/{styleFilePath}",
                assetsFilePath=f"../{staticSiteGenerator.assetsFilePath}",
                linkTo=linkTo,
                siteName=self.siteConfig.get("name", "No name attributed"),
                socialMediaList=social_media,
            )

            importedRuntime = {
                task.name: {
                    "display": "plot"
                    if task.arguments_label[0].isnumeric()
                    else "histo",
                    "status": task.get_status(target=libraryName),
                    "data": [
                        {
                            "arguments": float(arg) if arg.isnumeric() else arg,
                            "resultElement": res,
                            "libraryName": libraryName,
                        }
                        for arg, res in zip(
                            task.arguments_label,
                            task.mean_runtime(libraryName),
                        )
                        if res >= 0 and res != float("inf")
                    ],
                }
                for task in Library.GetLibraryByName(libraryName).tasks
            }
            # print(importedData)
            # CLASSEMENT DES LIBRAIRIES PAR TACHES
            HTMLLibraryRanking = staticSiteGenerator.CreateHTMLComponent(
                "library.html",
                libraryName=libraryName,
                taskNameList=[
                    (taskName, RemoveUnderscoreAndDash(taskName))
                    for taskName in Task.GetAllTaskName()
                ],
                scriptFilePath=BenchSite.CreateScriptBalise(
                    scriptName=f"../{staticSiteGenerator.scriptFilePath}/{scriptFilePath}",
                    module=True,
                ),
                scriptData=BenchSite.CreateScriptBalise(
                    content=f"const importedData = {importedRuntime};"
                ),
                taskDescription=libraryConfig[libraryName].get(
                    "description", "No Description Attributed"
                ),
                logoLibrary=f"<img src='../{logoLibrary[libraryName]}' alt='{libraryName}' width='50' height='50'>"
                if logoLibrary[libraryName] != None
                else "",
            )

            staticSiteGenerator.CreateHTMLPage(
                [
                    HTMLHeader,
                    HTMLNavigation,
                    HTMLGlobalRankingBar,
                    HTMLLibraryRanking,
                    HTMLGoogleAnalytics,
                    HTMLFooter,
                ],
                f"{libraryName}.html",
            )

        logger.info("=======Static site generated successfully=======")


if __name__ == "__main__":
    # création du site statique
    current_path = os.path.dirname(os.path.realpath(__file__))
    cwd = os.getcwd()
    benchSite = BenchSite("results.json")
    pagePath = "pages"
    benchSite.get_library_config()
    benchSite.get_task_config()
    benchSite.GenerateStaticSite()
    benchSite.get_theme_config()



import argparse
import os
import shutil
import subprocess
from pathlib import Path

from genbenchsite.src.benchmark import Benchmark
from genbenchsite.src.benchsite import BenchSite
from genbenchsite.src.logger import logger
from genbenchsite.src.template import create_template


def delete_directory(dir_path: str):
    """
    Clears the contents of a directory.

    Arguments
    ---------
    dir_path : str
        The path to the directory to clear.

    """
    logger.info(f"Deleting directory: {dir_path}")
    path = Path(dir_path)
    if path.exists() and path.is_dir():
        shutil.rmtree(path)
        logger.info(f"Deleted directory: {path}")
    else:
        logger.warning(f"Directory not found: {path}")


def delete_file(file_path):
    """
    Clears the contents of a file.

    Arguments
    ---------
    file_path : str
        The path to the file to clear.
    """
    logger.info(f"Deleting file: {file_path}")
    path = Path(file_path)
    if path.exists() and path.is_file():
        os.remove(path)
        logger.info(f"Deleted file: {path}")
    else:
        logger.warning(f"File not found: {path}")


def start_benchmark(structure_test_path: str, resultFilename: str = "results.json"):
    """
    Starts the benchmark script with the given parameters.

    Arguments
    ---------
    structure_test_path : str

    """
    baseFilename = resultFilename if Path(resultFilename).exists() else None
    benchmark = Benchmark(
        pathToInfrastructure=structure_test_path, baseResult=baseFilename
    )
    benchmark.StartAllProcedure()
    benchmark.ConvertResultToJson(outputFileName=resultFilename)

def benchmark(args):
    start_benchmark(
        structure_test_path=args.I,
        resultFilename='results.json',
    )

def website(args):
    BenchSite(
        inputFilename="results.json",
        outputPath=args.O,
        structureTestPath=args.I,
    ).GenerateStaticSite()


def create_name_for_backup_file(resultFilename: str):
    """
    Generates the name of the backup file.

    Arguments
    ---------
    resultFilename : str
        The path to the file containing the results of the benchmark.
    """
    import datetime

    # we get the current date
    now = datetime.datetime.now()
    # we get the name of the file
    filename = Path(resultFilename).name
    # we get the extension of the file
    extension = Path(resultFilename).suffix
    # we get the name of the file without the extension
    filename_without_extension = filename.replace(extension, "")
    # we create the name of the backup file
    backup_filename = (
        f"{filename_without_extension}_{now.strftime('%m-%d_%H-%M')}{extension}"
    )
    return backup_filename


def repository_is_local(repository, resultFilename: str = "results.json"):
    # we save the resutls file in a backup folder
    # as we can't know if the user want to keep the old results or not
    # if resultFilename.exists():
    #     BackupResultsFile(resultFilename)
    #     # we delete the results file
    #     delete_file(resultFilename)
    return Path(repository)


def get_task_to_reset(python_files_changed: list):
    """
    Get the tasks to reset from the python files changed.

    Arguments
    ---------
    python_files_changed : list
        The list of python files changed.
    """
    task_to_reset = []
    for file in python_files_changed:
        task_path = Path(file)
        task_to_reset.append(task_path.parent.name)
    logger.debug(f"Tasks to reset: {task_to_reset}")
    return task_to_reset


def save_results_in_backup(resultFilename: str):
    """
    Backup the results file.

    Arguments
    ---------
    resultFilename : str
        The path to the file containing the results of the benchmark.
    """
    backup_filename = create_name_for_backup_file(resultFilename)
    # we create the backup folder
    backup_folder = Path("backup")
    if not backup_folder.exists():
        backup_folder.mkdir()
    # we copy the results file in the backup folder
    shutil.copyfile(
        resultFilename,
        backup_folder / backup_filename,
    )
    logger.info(
        f"Backup of the previous results file {resultFilename} created in {backup_folder} folder with the name {backup_filename}"
    )


def reset_task(task_to_reset: list, resultFilename: str = "results.json"):
    """
    Reset the tasks.

    Arguments
    ---------
    task_to_reset : list
        The list of tasks to reset.
    """
    import json

    resultFilename = Path(resultFilename)
    # we check if the file exists
    if not resultFilename.exists():
        logger.warning("No results file to reset, you must run the benchmark first")
        logger.warning(f"File {resultFilename} does not exist")
        return
    # we open the file
    results = None
    with open(resultFilename, "r") as f:
        results = json.load(f)
    # we check if the file is empty
    if len(results) == 0:
        logger.warning("No results file to reset, the file is empty")
        logger.warning(f"File {resultFilename} is empty")
        return
    # we reset the tasks
    for target in results.keys():
        for task in task_to_reset:
            if task in results[target].keys():
                results[target][task]["results"] = {}
            else:
                logger.warning(f"Task {task} not found in results file")
    # we save the file
    with open(resultFilename, "w") as f:
        json.dump(results, f, indent=4)


def repository_is_github(repository, resultFilename: str = "results.json"):
    default_repository_name = "repository"

    # we create a local repository
    path = Path(default_repository_name)
    if not path.exists():
        logger.debug(f"Creating the local repository {path}")
        path.mkdir()
    else:
        # we check if a python file has changed since the last pull
        python_files_changed = has_python_file_changed(path.absolute().__str__())
        if len(python_files_changed) > 0:
            logger.info(f"Python file has changed since the last pull")
            # we clear the local repository and the results file needed for the benchmark
            # as the old test are now deprecated we delete the old results
            # but before that, we create a backup of the results in case the user want to keep them
            if resultFilename.exists():
                save_results_in_backup(resultFilename)
                # we get the tasks to reset
                task_to_reset = get_task_to_reset(python_files_changed)
                # we reset the tasks
                reset_task(task_to_reset, resultFilename=resultFilename)
                # we delete the local repository
                delete_directory(path.absolute().__str__())
                delete_file(resultFilename)

        else:
            logger.info(f"No python file has changed since the last pull")
            # we merge the remote repository with the local repository
            command = f"git -C {path} pull"
            try:
                os.system(command)
            except:
                logger.error(
                    f"Error when merging the remote repository with the local repository {repository}"
                )
                raise Exception(
                    f"Error when merging the remote repository with the local repository {repository}"
                )
            return path

    # we clone the repository in the local repository
    command = f"git clone {repository} {path}"
    try:
        os.system(command)
    except:
        logger.error(f"Error when cloning the repository {repository}")
        raise Exception(f"Error when cloning the repository {repository}")

    return path


def has_python_file_changed(repository_name: str) -> list:
    # we memorize the current directory
    current_dir = os.getcwd()
    # we check if a python file has changed since the last pull
    # we go to the repository
    os.chdir(repository_name)
    # we get the last commit
    fetch = subprocess.Popen(["git", "fetch"], stdout=subprocess.PIPE)
    fetch.wait()
    # we get the last commit of the remote repository
    last_commit = subprocess.check_output(["git", "rev-parse", "FETCH_HEAD"]).strip()
    # we get the last commit of the local repository
    local_last_commit = subprocess.check_output(["git", "rev-parse", "HEAD"]).strip()
    # we compare the two commits
    # compare the SHAs to see if the Python file has changed
    diff_output = subprocess.check_output(
        ["git", "diff", "--name-status", last_commit, local_last_commit],
        encoding="utf-8",
    ).splitlines()
    logger.debug(f"Diff output: {diff_output}")
    # we go back to the current directory
    os.chdir(current_dir)
    changed_python_files = []
    for line in diff_output:
        status, file = line.split("\t")
        if file.endswith(".py"):
            if status == "A":
                changed_python_files.append(file)
                logger.info(f"Python file {file} has been added")
            elif status == "D":
                changed_python_files.append(file)
                logger.info(f"Python file {file} has been deleted")
            elif status == "M":
                changed_python_files.append(file)
                logger.info(f"Python file {file} has been modified")
    return changed_python_files


def enough_test_to_publish(resultFilename: str, min_test_required: int = 10):
    """
    Checks if there are enough tests to publish the results.

    Arguments
    ---------
    resultFilename : str
        The path to the file containing the results of the benchmark.
    """
    # we check if the file exists
    if not Path(resultFilename).exists():
        return False
    # we check if the file is empty
    if Path(resultFilename).stat().st_size == 0:
        return False
    # we're not constraint by the number of tests
    if min_test_required == 0:
        return True
    # we check if there are enough tests
    return count_test() % min_test_required == 0


def count_test():
    import json_to_python_object as jtpo

    return jtpo.count_test()


def init(args):
    create_template(args.benchmark_name, args.nb_targets, args.nb_themes, args.nb_tasks)


def reset(args):
    task_name = args.task_name
    # we check if the task name is not empty
    if task_name is None or len(task_name) == 0:
        logger.error("No task name provided")
        raise Exception("No task name provided")
    # we reset the tasks
    logger.debug(f"Tasks to reset: {task_name}")
    reset_task(task_name)


def auto(args):
    

    # Test the repository
    result_filename = Path("results.json")

    # by default if the user provide the url, we clone the repository in the local repository
    # the online repository takes precedence over the local repository if the user provide both (but it's not recommended)

    # if args.url is not None:
    #     logger.info(f"Cloning the repository {args.url}")
    #     local_directory = repository_is_github(args.url, resultFilename=resultFilename.absolute())
    # else:
    #     logger.info(f"Using the local repository {args.O}")
    #     local_directory = repository_is_local(args.O, resultFilename=resultFilename.absolute())

    local_directory = repository_is_github(
        args.url, resultFilename=result_filename.absolute()
    )

    if not local_directory.exists():
        logger.error(f"Path {local_directory.absolute()} does not exist")
        raise Exception(f"Path {local_directory.absolute()} does not exist")

    
    start_benchmark(
        local_directory.absolute().__str__(),
        result_filename.absolute().__str__(),
    )
        

    # The second step is to create the HTML page from the test results. This HTML page will be
    # created in the output folder. The output folder is the folder where the user want to save the
    # HTML page. The output folder is the same as the input folder if the user didn't specify an output folder.
    
    benchsite = BenchSite(
        inputFilename=result_filename.absolute().__str__(),
        outputPath=Path().cwd() / "website",
        structureTestPath=local_directory.absolute().__str__(),
    )
    benchsite.GenerateStaticSite()

    # we copy the result.json file in the output folder
    shutil.copyfile(
        result_filename.absolute(),
        os.path.join(args.output_folder, result_filename),
    )

    # The third step is to deploy the HTML page on a server. The server is a github page. The user
    # must have a github account and a github repository. The user must have a github token to deploy
    # the HTML page on the github page. The user must specify the name of the github repository where
    # the HTML page will be deployed.
    return
    if args.publish and args.url is not None:
        if not args.force_publish and not enough_test_to_publish(
            result_filename.absolute().__str__()
        ):
            logger.info("Not enough tests to publish the results")
            logger.debug(f"Number of tests: {count_test()}")
            exit(0)
        logger.info("Publishing the HTML page on the github page")
        # before copying the output folder in the repository, we need to check if there is not already
        # copy the output folder in the repository
        if os.path.exists(os.path.join(local_directory.absolute(), args.output_folder)):
            logger.info(
                f"Removing the folder {os.path.join(local_directory.absolute(), args.output_folder)}"
            )
            shutil.rmtree(os.path.join(local_directory.absolute(), args.output_folder))
        shutil.copytree(
            args.output_folder,
            os.path.join(local_directory.absolute(), args.output_folder),
        )
        os.chdir(local_directory.absolute())
        os.system(f"git add {args.output_folder}")
        os.system(f'git commit -m "Updating the HTML page"')
        os.system(f"git push")
        logger.info("HTML page deployed on the github page")

        # we remove the local repository
        # shutil.rmtree(working_directory.absolute())


def main():
    # first step is to Run the tests and evaluate the library based on the repository
    # inputed by the user. This repository may be a github repository or a local repository.

    # usage :
    # python main.py <repository> [-A <access_folder>] [-O <output_folder>] [-P <publish>] [-B <benchmark>]

    # access_folder = the way the repository is accessed. If the repository is local, the value is local. If the repository is on github, the value is github
    # repository = the path of the repository if isLocal is True, the name of the repository if isLocal is False
    # output_folder = the path of the folder where the user want to save the HTML page
    # publish = True if the user want to deploy the HTML page, False otherwise

    # current_dir = os.path.dirname(os.path.abspath(__file__))
    # # we go to the current directory
    # os.chdir(current_dir)

    parser = argparse.ArgumentParser(
        description="Generate a static website of a benchmark of libraries."
    )

    subparsers = parser.add_subparsers(dest="action", help="The action to perform")

    # Subparser for the init action
    init_parser = subparsers.add_parser("init", help="Initialize the repository")
    init_parser.add_argument(
        "benchmark_name", type=str, help="The name of the repository"
    )
    init_parser.add_argument(
        "-nb_targets", type=int, help="The number of targets", default=2
    )
    init_parser.add_argument(
        "-nb_themes", type=int, help="The number of themes", default=2
    )
    init_parser.add_argument(
        "-nb_tasks", type=int, help="The number of tasks", default=2
    )
    init_parser.set_defaults(func=init)

    # Subparser for the reset action
    reset_parser = subparsers.add_parser("reset", help="Reset the repository")
    reset_parser.add_argument(
        "task_name",
        type=str,
        help="The name of the tasks to reset (separated by a space)",
        nargs="+",
    )
    reset_parser.set_defaults(func=reset)

    # Subparser for the benchmark action
    benchmark_parser = subparsers.add_parser("benchmark", help="Run the benchmark tests")
    benchmark_parser.add_argument(
        "-I",
        type=str,
        help="The local path of the folder where the benchmark structure is located",
        default="repository",
    )
    benchmark_parser.set_defaults(func=benchmark)

    # Subparser for the website action
    website_parser = subparsers.add_parser("website", help="Generate the website")
    website_parser.add_argument(
        "-I",
        type=str,
        help="The local path of the folder where the benchmark structure is located",
        default="repository",
    )
    website_parser.add_argument(
        "-O",
        type=str,
        help="The local path of the folder where the HTML page will be generated",
        default="website",
    )
    website_parser.set_defaults(func=website)

    # Subparser for the auto action
    auto_parser = subparsers.add_parser("auto", help="Run the benchmark tests")
    auto_parser.add_argument(
        "url",
        type=str,
        help="The url of the git repository where the benchmark structure is located",
    )
    # auto_parser.add_argument(
    #     "--publish",
    #     help="True if the user want to deploy the HTML page, False otherwise",
    #     default=True,
    #     action=argparse.BooleanOptionalAction,
    # )
    # auto_parser.add_argument(
    #     "--benchmark",
    #     help="True if the user want to run the benchmark, False otherwise. If set to False, the user must provide the result.json file in the repository",
    #     default=True,
    #     action=argparse.BooleanOptionalAction,
    # )
    auto_parser.set_defaults(func=auto)

    args = parser.parse_args()
    logger.info(f"Arguments: {args}")
    logger.info(f"Action: {args.action}")

    logger.info("=======Starting the main script=======")

    # Call the appropriate function based on the selected action
    if args.action is None:
        parser.print_help()
        return
    args.func(args)


if __name__ == "__main__":
    main()



"""Docstring for Library.py module.

This module contains the class Library and the differents function to manipulate the Library class.

"""

from dataclasses import dataclass, field
from typing import ClassVar

from genbenchsite.src.task import Task


@dataclass
class Library:
    """Store all the information about all the libraries created.

    Attributes
    ----------
    name : str
        The name of the library.
    tasks : list of Task
        The list of the tasks of the library.
    allLibrary : list of Library
        Class Attribute ! The list of all the libraries created.

    """

    name: str
    tasks: list[Task] = field(default_factory=list)
    allLibrary: ClassVar[list["Library"]] = []

    def __post_init__(self) -> None:
        self.allLibrary.append(self)

    def __repr__(self) -> str:
        return f"Library({self.name})"

    @classmethod
    def GetAllLibrary(cls) -> list["Library"]:
        """Getter for all the libraries created.

        Returns
        -------
        list of Library
            The list of all the libraries created.
        """
        return cls.allLibrary

    @classmethod
    def GetAllLibraryName(cls):
        """Getter for all the libraries name created.

        Returns
        -------
        list of str
            The list of all the libraries name created.
        """
        return (library.name for library in cls.allLibrary)

    @classmethod
    def GetLibraryByTaskName(cls, taskName: str) -> list["Library"]:
        """Getter for all the libraries that contains a task with the name taskName.

        Parameters
        ----------
        taskName : str
            The name of the task to search.

        Returns
        -------
        list of Library
            The list of all the libraries that contains a task with the name taskName.
        """
        libraryList = []
        for library in cls.GetAllLibrary():
            libraryList.extend(
                [library for task in library.tasks if task.name == taskName]
            )
        return libraryList

    @classmethod
    def GetLibraryByName(cls, libraryName: str) -> "Library":
        """Getter for the library with the name libraryName.

        Parameters
        ----------
        libraryName : str
            The name of the library to search.

        Returns
        -------
        Library
            The library with the name libraryName.
        """
        for library in cls.allLibrary:
            if library.name == libraryName:
                return library
        return None

    def GetTaskByName(self, taskName: str) -> Task:
        """Getter for the task with the name taskName in the library.

        Parameters
        ----------
        taskName : str
            The name of the task to search.

        Returns
        -------
        Task
            The task with the name taskName in the library.
        """
        for task in self.tasks:
            if task.name == taskName:
                return task
        return None

    @classmethod
    def GetTaskByLibraryNameAndTaskName(cls, libraryName: str, taskName: str) -> "Task":
        """Getter for the task for a specific name of library and name of task.

        Parameters
        ----------
        libraryName : str
            The name of the library to search.
        taskName : str
            The name of the task to search.

        Returns
        -------
        Task
            The task with the name `taskName` in the library with the name `libraryName`.
        """
        return cls.GetLibraryByName(libraryName).GetTaskByName(taskName)

    @classmethod
    def GetResultsByTaskName(cls, taskName: str) -> dict[str, list[float]]:
        """Getter for the results of all the tasks with the name `taskName`.

        Parameters
        ----------
        taskName : str
            The name of the task to search.

        Returns
        -------
        dict of str and list of float
            The results of all the tasks with the name `taskName`. The key is the name of the library and the value is the list of the results.
        """
        dico = {}
        for library in cls.GetAllLibrary():
            task = library.GetTaskByName(taskName)
            dico[library.name] = task.get_calculated_runtime(library.name)
        return dico

    @classmethod
    def GetArgumentsByTaskName(cls, taskName: str) -> dict[str, list[float]]:
        """Getter for the arguments of all the tasks with the name `taskName`.

        Parameters
        ----------
        taskName : str
            The name of the task to search.

        Returns
        -------
        dict of str and list of float
            The arguments of all the tasks with the name `taskName`. The key is the name of the library and the value is the list of the arguments.
        """
        dico = {}
        for library in cls.GetAllLibrary():
            task = library.GetTaskByName(taskName)
            dico[library.name] = task.arguments
        return dico



from pygments import highlight
from pygments.lexers import PythonLexer
from pygments.formatters import HtmlFormatter
import json
from pathlib import Path
from genbenchsite.src.logger import logger


class CollectCode:
    def __init__(self, pathToInfrastructure: str, outputPath=None):
        logger.info("Collecting the code")
        logger.debug(f"Path to infrastructure : {pathToInfrastructure}")
        self.pathToInfrastructure = Path(pathToInfrastructure)

        self.taskNames = []

        self.targets = [
            path.name for path in self.pathToInfrastructure.glob("targets/*")
        ]

        self.taskPath = list(self.pathToInfrastructure.glob("**/*_run.py"))

        logger.debug(f"Task path : {self.taskPath}")
        logger.debug(f"Targets : {self.targets}")

        self.pure_code_str = self.RetreiveCode(*self.taskPath)

        self.CodeHTML = {target: {} for target in self.targets}

        self.TransfomCodeInHTML()
        logger.info("=======Code collected=======")

    def RetreiveCode(self, *code_path):
        if len(code_path) == 0:
            logger.warning("No path given")
            return {}

        code = {target: {} for target in self.targets}
        for path in code_path:
            # we check if there is a before in the pathName
            # maybe change strategy in the future to be more flexible
            if path.name.split("_")[1] == "before":
                continue
            taskName = path.parent.name
            targetName = path.name.split("_")[0]
            logger.debug(f"Reading code file in {path.absolute()}")
            with open(path.absolute(), "r") as f:
                code[targetName][taskName] = f.read()

        logger.info("Code retreived")

        return code

    def TransfomCodeInHTML(self):
        for target in self.targets:
            for task in self.pure_code_str[target]:
                self.CodeHTML[target][task] = self.pure_code_to_html(
                    self.pure_code_str[target][task]
                )
        logger.info("Code transformed in HTML")
        logger.debug(f"Code HTML : {self.CodeHTML.keys()}")

    def pure_code_to_html(self, code: str):
        formatter = HtmlFormatter(
            linenos=True,
            cssclass="zenburn",
            noclasses=True,
            style="zenburn",
        )
        return highlight(code, PythonLexer(), formatter)

    def SaveInJson(self, outputPath: str):
        with open(outputPath, "w") as file:
            json.dump(self.CodeHTML, file)

    def get_code_HTML(self, target, task):
        return self.CodeHTML[target].get(task, None)


if __name__ == "__main__":
    pathToInfrastructure = "C:/Users/jules/Documents/Git/BenchSite/repository"

    collectCode = CollectCode(pathToInfrastructure)



"""Docstring for the module static_site_generator.py

This module contains the class StaticSiteGenerator and the differents function to manipulate the StaticSiteGenerator class.
You can use it to create and generate static html page with jinja2.

"""

from jinja2 import Environment, FileSystemLoader
import os
import shutil
from pathlib import Path


class StaticSiteGenerator:
    """
    Create the static site generator object.

    Attributes
    ----------
    scriptFilePath : str
        The path of the folder where the images are stored.
    htmlTemplateFilePath : str
        The path of the folder where the HTML templates are stored.
    assetsFilePath : str
        The path of the folder where the assets are stored.
    contentFilePath : str
        The path of the folder where the output files are stored.
    styleFilePath : str
        The path of the folder where the CSS files are stored.
    """

    def __init__(
        self,
        output_website_path: str = "website",
    ):
        """Create the static site generator object

        Check if the path exist, if not, create the folder if `createFolder` is True.

        Parameters
        ----------
        scriptFilePath : str
            The path of the folder where the script are stored, by default "script"
        htmlTemplateFilePath : str
            The path of the folder where the HTML templates are stored.
        assetsFilePath : str
            The path of the folder where the assets are stored.
        contentFilePath : str
            The path of the folder where the output files are stored.
        styleFilePath : str
            The path of the folder where the CSS files are stored.
        createFolder : bool, optional
            If True, create the folder if it does not exist, by default True
        """

        # the path of the script
        website_template_path = Path(__file__).parent.parent / "website_template"
        html_components_path = Path(__file__).parent.parent / "html_template"
         # first we make a copy of the website_template folder in the output folder
        self.output_website_path = Path(output_website_path)
        # if the output_website_path does exist, we delete it to create a updated version
        if self.output_website_path.exists():
            delete_directory(self.output_website_path)

        # we copy the website_template folder in the output_website_path
        shutil.copytree(website_template_path, self.output_website_path)

        # we create the content folder
        self.contentFilePath = self.output_website_path / "content"
        self.contentFilePath.mkdir()

        # we now link the differents attributes to the path of the website_template folder
        self.scriptFilePath = self.output_website_path / "script"
        self.assetsFilePath = self.output_website_path / "assets"
        self.styleFilePath = self.output_website_path / "style"
        self.contentFilePath = self.output_website_path / "content"
        self.htmlTemplateFilePath = html_components_path


        # for path in [
        #     scriptFilePath,
        #     htmlTemplateFilePath,
        #     assetsFilePath,
        #     contentFilePath,
        #     styleFilePath,
        # ]:
        #     if not self.CheckIfPathExist(path) and not createFolder:
        #         raise Exception(f"Path {path} does not exist")
        #     elif not self.CheckIfPathExist(path) and createFolder:
        #         # Create the folder relative to path of the script
        #         os.mkdir(os.path.join(current_path, path))

        # # we clean the output folder
        # for file in os.listdir(os.path.join(current_path, contentFilePath)):
        #     os.remove(os.path.join(current_path, contentFilePath, file))

        # we need the basename of these path to use it in the HTML template
        basename = lambda path: os.path.basename(os.path.normpath(path))

        self.scriptFilePath = basename(self.scriptFilePath)
        self.assetsFilePath = basename(self.assetsFilePath)
        self.styleFilePath = basename(self.styleFilePath)



    def CheckIfPathExist(self, path: str) -> bool:
        """Check if the path relative to the path of the script exist.

        Parameters
        ----------
        path : str
            The path to check

        Returns
        -------
        bool
            True if the path exist, False otherwise
        """

        return os.path.exists(os.path.join(os.path.dirname(__file__), path))

    def CreateHTMLComponent(self, templateName: str, **kwargs) -> str:
        """Create the HTML component from the template and the arguments. The arguments must be in the form of a dictionary.
        the key of the dictionary will be the name of the variable in the template.

        Parameters
        ----------
        templateName : str
            The name of the template to use.
        **kwargs : dict
            The arguments to pass to the template.

        Returns
        -------
        str
            The HTML component

        """
        file_loader = FileSystemLoader(self.htmlTemplateFilePath)
        env = Environment(loader=file_loader)
        template = env.get_template(templateName)
        return template.render(**kwargs)

    def CreateHTMLPage(
        self, HTMLComponent: list[str], pageName: str, manualOutputPath=None
    ) -> None:
        """Create the HTML page from the HTML component list. You can compose the HTML component list as you want,
        but the order is important. The first element of the list will be the first element of the HTML page.

        Parameters
        ----------
        HTMLComponent : list
            The list of the HTML component to use.
        pageName : str
            The name of the page to create.

        """
        html = "".join(HTMLComponent)
        outputPath = self.contentFilePath
        if manualOutputPath is not None:
            outputPath = manualOutputPath

        with open(f"{outputPath}/{pageName}", "w") as f:
            f.write(html)

def delete_directory(dir_path: str):
    """
    Clears the contents of a directory.

    Arguments
    ---------
    dir_path : str
        The path to the directory to clear.

    """
    path = Path(dir_path)
    if path.exists() and path.is_dir():
        shutil.rmtree(path)
    else:
        print(f"Directory {dir_path} does not exist.")

if __name__ == "__main__":
    ssg = StaticSiteGenerator()
    


"""Docstring for json_to_python_object.py.

This module contains the function to read a json file and create the python object.

"""
import os
import json
from genbenchsite.src.library import Library
from genbenchsite.src.task import Task
from genbenchsite.src.logger import logger
from genbenchsite.src.structure_test import StructureTest


def FileReaderJson(filename: str, structure_test_path: str) -> None:
    """Read a json file and create the python object.

    Parameters
    ----------
    filename : str
        The name of the json file.

    Returns
    -------
    tuple[list[Library], list[Task]]
        A tuple with a list of library and a list of task.

    """
    data = readJsonFile(filename)
    strtest = StructureTest()
    taskConfig = strtest.read_config(
        *strtest.find_config_file(os.path.join(structure_test_path, "themes"))
    )

    for libName, libInfo in data.items():
        library = Library(libName)
        for taskName, taskInfo in libInfo.items():
            task = (
                Task(taskName, taskInfo["theme"])
                if taskName not in Task.GetAllTaskName()
                else Task.GetTaskByName(taskName)
            )
            logger.info(f"Task {taskName} with {libName} library")
            logger.debug(f"arguments: {len(taskInfo['results'].keys())}")

            task.arguments_label = [argument for argument in taskInfo["results"].keys()]
            # transform the argument label into a list of index to be able to use the LexMax algorithm
            task.arguments.extend(TokenizeArguments(task.arguments_label))

            runtime = [
                taskInfo["results"].get(argument).get("runtime")
                for argument in task.arguments_label
            ]
            evaluation = [
                taskInfo["results"].get(argument).get("evaluation")
                for argument in task.arguments_label
            ]
            if evaluation[0] is None:
                evaluation = None

            task.runtime[libName] = runtime
            task.evaluation[libName] = evaluation
            evaluation_function_name = taskConfig[task.name].get("evaluation_function")
            if evaluation_function_name:
                task.evaluation_function_name = evaluation_function_name.split(" ")
                task.evaluation_sort_order = taskConfig[task.name].get(
                    "evaluation_sort_order"
                )
                if task.evaluation_sort_order:
                    task.evaluation_sort_order = task.evaluation_sort_order.split(",")
                else:
                    task.evaluation_sort_order = [
                        "desc" for _ in range(len(task.evaluation_function_name))
                    ]

            library.tasks.append(task)


def TokenizeArguments(arguments: list[str]) -> list[int]:
    return [index for index, _ in enumerate(arguments)]


def readJsonFile(filename: str):
    try:
        with open(filename, "r") as file:
            data = json.load(file)
    except FileNotFoundError:
        logger.error(f"File {filename} not found")
        data = None

    return data


def count_test():
    # we tak a task at random
    task = Task.allTasks[0]
    # we take a library at random
    library = Library.allLibrary[0]
    # we take the first runtime of the task
    runtime = task.runtime[library.name][0]
    return len(runtime)


if __name__ == "__main__":
    FileReaderJson("results.json")
    # FileReaderJson("essais.json")

    # print(Task.GetAllTaskName())

    # print(list(Library.GetAllLibraryName()))

    tasks = Task.allTasks
    for task in tasks:
        # print(task.name)
        # print(task.get_calculated_runtime("pgmpy"))
        # print(task.mean_runtime('pgmpy'))
        # print(task.get_calculated_evaluation("pyAgrum"))
        # print(task.get_calculated_evaluation("pgmpy"))
        # print(task.get_standard_deviation(task.runtime["pgmpy"]))
        # print(task.get_runtime("pgmpy"))
        # print(task.standard_deviation("pgmpy"))
        # print(task.standard_deviation_evaluation("pgmpy"))

        # task.mean_runtime('pyAgrum')
        # print(task.get_evaluation("pyAgrum"))
        # print(task.mean_evaluation("pyAgrum"))
        # print(task.standard_deviation_evaluation("pyAgrum"))
        # print(task.mean_evaluation("pyAgrum"))

        # print(task.standard_deviation_evaluation("pyAgrum"))
        # print(task.standard_deviation_evaluation("pyAgrum"))
        # print(task.standard_deviation_evaluation("pyAgrum"))
        pass



from setuptools import setup, find_packages

# load the README file.
with open("README.md", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="genbenchsite",
    version="0.0.2",
    description="Generate a benchmark website from a set of benchmark tests",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Jules Cassan",
    entry_points={
        "console_scripts": [
            "gbs = genbenchsite.src.main:main",
        ]
    },
    package_dir={"": "app"},
    packages=find_packages(where="app"),
    package_data={"": ["html_template/*", "template.txt", "website_template/**"]},
    python_requires=">=3.9",
    install_requires=[
        "colorama>=0.4.6",
        "Jinja2>=3.1.2",
        "MarkupSafe>=2.1.2",
        "numpy>=1.24.2",
        "psutil>=5.9.5",
        "Pygments>=2.15.1",
        "tqdm>=4.65.0",
        "rich>=12.5.1",
    ],
    extras_require={
        "dev": [
            "twine>=4.0.2",
        ],
    },
)



"""Docstring for task.py module.

This module contains the class Task and the differents function to manipulate the Task class.

"""

from dataclasses import dataclass, field
from typing import ClassVar
import numpy as np
from genbenchsite.src.logger import logger

MIN_RUNTIME_POSSIBLE = 0.0001


@dataclass
class Task:
    """
    Store all the information about all the tasks created.

    Attributes
    ----------
    name : str
        The name of the task.
    theme : str
        The theme of the task.
    arguments : list of float
        The list of the arguments of the task. The index of the argument correspond to the index of the result.
    results : list of float
        The list of the results of the task. The index of the result correspond to the index of the argument.
    allTasks : list of Task
        Class Atribute ! The list of all the tasks created.

    """

    name: str
    theme: str
    arguments: list[float] = field(default_factory=list)
    runtime: dict[str, list[float]] = field(default_factory=dict)
    evaluation: dict[str, list[float]] = field(default_factory=dict)
    evaluation_function_name: list[str] = field(default_factory=list)
    evaluation_sort_order: list[str] = field(default_factory=list)
    arguments_label: list[str] = field(default_factory=list)
    cache_runtime: dict[str, list[float]] = field(default_factory=dict)
    cache_evaluation: dict[str, list[float]] = field(default_factory=dict)
    allTasks: ClassVar[list["Task"]] = []

    def __post_init__(self) -> None:
        logger.debug(f"Task {self.name} created")
        Task.allTasks.append(self)

    def __repr__(self) -> str:
        return f"Task({self.name})-> arguments: {self.arguments_label}"

    @classmethod
    def GetAllTask(cls) -> list["Task"]:
        """Getter for all the tasks created.

        Returns
        -------
        list of Task
            The list of all the tasks created.

        """
        return cls.allTasks

    @classmethod
    def GetAllTaskName(cls) -> list[str]:
        """Getter for all the tasks name created.

        Returns
        -------
        listtaskName : list of str
            The list of all the tasks name created.

        """
        listTaskName = []
        for task in cls.allTasks:
            if task.name not in listTaskName:
                listTaskName.append(task.name)
        return listTaskName

    @classmethod
    def GetTaskByName(cls, taskName: str) -> "Task" or None:
        """Getter for a task by its name.

        Parameters
        ----------
        taskName : str
            The name of the task to get.

        Returns
        -------
        task : Task
            The task with the name given in parameter.

        """
        for task in cls.allTasks:
            if task.name == taskName:
                return task
        return None

    @classmethod
    def GetAllTaskByName(cls, taskName: str):
        """Getter for all the tasks with the same name.

        Parameters
        ----------
        taskName : str
            The name of the task to get.

        Returns
        -------
        list of Task
            The list of all the tasks with the same name.

        """
        return (task for task in cls.allTasks if task.name == taskName)

    @classmethod
    def GetAllThemeName(cls):
        """Getter for all the theme name created.

        Returns
        -------
        listThemeName : list of str
            The list of all the theme name created.

        """
        listThemeName = []
        for task in cls.GetAllTask():
            if task.theme not in listThemeName:
                listThemeName.append(task.theme)
        return listThemeName

    @classmethod
    def GetTaskByThemeName(cls, themeName: str):
        """Getter for all the tasks with the same theme name.

        Parameters
        ----------
        themeName : str
            The name of the theme to get.

        Returns
        -------
        list of Task
            The list of all the tasks with the same theme name.

        """
        return (task for task in cls.GetAllTask() if task.theme == themeName)

    @classmethod
    def GetTaskNameByThemeName(cls, themeName: str) -> list[str]:
        """Getter for all the tasks name with the same theme name.

        Parameters
        ----------
        themeName : str
            The name of the theme to get.

        Returns
        -------
        list of str
            The list of all the tasks name with the same theme name.

        """
        listTaskName = []
        for task in cls.GetTaskByThemeName(themeName):
            if task.name not in listTaskName:
                listTaskName.append(task.name)
        return listTaskName

    @staticmethod
    def str_and_none_to_nan(array: np.ndarray) -> np.ndarray:
        """transform the string and None into np.nan inside an array and transform the array into float64"""

        def is_float(string: str):
            """return True if the string is a float, False otherwise"""
            try:
                float(string)
                return True
            except ValueError:
                return False

        return np.where(
            np.vectorize(lambda x: x is None or not is_float(x))(array), np.nan, array
        ).astype(np.float64)

    def get_runtime(self, target: str) -> list[float]:
        # for element in self.runtime[target]:
        #     print(len(element))
        #     print(element)
        # print(self.runtime[target])
        # we transform the string and None into np.nan and transform the array into float64
        runtime = Task.str_and_none_to_nan(np.array(self.runtime[target]))
        # if there is no runtime for the target, we return a list of np.nan with the same size as the arguments
        if (np.isnan(runtime)).all():
            # problem with the shape of the array
            return np.vstack(runtime).tolist()
            # return np.hstack(runtime).tolist()
        # we inverse the runtime to have the difference between the end and the start (end - start)
        runtime = np.hstack(np.diff(runtime, axis=2)).T
        # we adapt the value if the runtime is negative
        runtime = np.where(runtime <= 0, MIN_RUNTIME_POSSIBLE, runtime)
        logger.debug(f"Runtime for {target} in {self.name} : {runtime}")
        return runtime.tolist()

    def get_evaluation(self, target: str) -> list[float]:
        # if there is no evaluation for the target, we return a list of np.nan with the same size as the arguments
        if len(self.evaluation_function_name) == 0:
            evaluation = [float("inf")] * len(self.arguments_label)
            logger.debug(f"Evaluation for {target} in {self.name} : {evaluation}")
            return evaluation

            # if self.evaluation[target] is None:
            #     # the evaluation is a error message
            #     evaluation = [float("inf")] * len(self.arguments_label)
            #     logger.debug(f"Evaluation for {target} in {self.name} : {evaluation}")
            return evaluation

        evaluation = self.evaluation[target][:]
        for i in range(len(evaluation)):
            for function in evaluation[i].keys():
                # we transform the string and None into np.nan and transform the array into float64 to be able to do computation
                evaluation[i][function] = Task.str_and_none_to_nan(
                    evaluation[i][function]
                )
                # if the evaluation is a list of string or None, that means that all the evaluation failed
                if np.isnan(evaluation[i][function]).all():
                    evaluation[i][function] = float("inf")
                else:
                    evaluation[i][function] = evaluation[i][function].tolist()
        return evaluation

    def mean_runtime(self, target: str) -> list[float]:
        if target in self.cache_runtime:
            logger.debug(
                f"Evaluation already calculated for {target} in {self.name}, using the cached value"
            )
            return self.cache_runtime[target]
        runtime = self.get_runtime(target)
        # print(runtime)
        runtime = np.nanmean(runtime, axis=1)
        runtime[np.isnan(runtime)] = float("inf")
        logger.debug(f"Runtime for {target} in {self.name} : {runtime}")
        # we save the runtime in the cache
        self.cache_runtime[target] = runtime.tolist()
        return runtime.tolist()

    def mean_evaluation(self, target: str) -> list[float]:
        if target in self.cache_evaluation:
            logger.debug(
                f"Evaluation already calculated for {target} in {self.name}, using the cached value"
            )
            return self.cache_evaluation[target]
        evaluation = []
        for element in self.get_evaluation(target):
            if element == float("inf"):
                evaluation.append(element)
                continue
            evaluation.append(element.copy())
        for i in range(len(evaluation)):
            if evaluation[i] == float("inf"):
                continue
            for function in evaluation[i].keys():
                evaluation[i][function] = np.nanmean(evaluation[i][function]).tolist()
                if np.isnan(evaluation[i][function]):
                    evaluation[i][function] = float("inf")
        logger.debug(f"Evaluation for {target} in {self.name} : {evaluation}")
        # we save the evaluation in the cache
        self.cache_evaluation[target] = evaluation
        return evaluation

    def standard_deviation_runtime(self, target) -> list[float]:
        std_runtime = np.nanstd(self.get_runtime(target=target), axis=1)
        std_runtime = np.where(std_runtime <= 0, MIN_RUNTIME_POSSIBLE, std_runtime)
        logger.debug(f"Standard deviation for {target} in {self.name} : {std_runtime}")
        return std_runtime.tolist()

    def standard_deviation_evaluation(self, target) -> list[float]:
        evaluation = []
        for element in self.get_evaluation(target):
            if element == float("inf"):
                evaluation.append(element)
                continue
            evaluation.append(element.copy())
        # evaluation = self.get_evaluation(target)[:]
        for i in range(len(evaluation)):
            if evaluation[i] == float("inf"):
                continue
            for function in evaluation[i].keys():
                evaluation[i][function] = np.nanstd(evaluation[i][function]).tolist()
        return evaluation

    def variance(self, target) -> list[float]:
        return np.nanvar(self.get_runtime(target=target), axis=1).tolist()

    def get_status(self, target: str) -> str:
        """Getter for the status of the task.

        Returns
        -------
        status : str
            The status of the task.

        """
        mean = np.array(self.mean_runtime(target))
        if (mean == float("inf")).all():
            if isinstance(self.runtime[target][0], str):
                # the runtime is a error message
                return self.runtime[target][0]
            else:
                # we want to find the first error message
                return self.runtime[target][0][0][1]
        return "Run"


if __name__ == "__main__":
    from json_to_python_object import FileReaderJson

    FileReaderJson("results.json")

    print(Task.GetAllTaskName())



import os
import subprocess
import time
import json
import numpy as np
import ast
from tqdm import tqdm
from pathlib import Path

from genbenchsite.src.logger import logger
from genbenchsite.src.structure_test import StructureTest


class Benchmark:
    """
    Benchmark is a class that run process for each library and each task and save the results in a json file

    We expect a very specific infrastucture of file and folder in order to run the test. The infrastucture is the following:
    - pathToInfrastructure
        - targets
            - config.ini
        - tasks
            - task1
                - config.ini
                - [<beforeBuildScript>]
                - [<libraryScript>] *
                - [file(data)]
            - task2
            [...]

    Class Attributes
    ----------
    NOT_RUN_VALUE : str or int
        value that will be used in the json file if the task has not been run
    ERROR_VALUE : str or int
        value that will be used in the json file if an error occured during the task
    """

    NOT_RUN_VALUE = "NotRun"
    ERROR_VALUE = "Error"
    DEFAULT_VALUE = "Infinity"
    TIMEOUT_VALUE = "Timeout"

    DEFAULT_TIMEOUT = 40
    DEFAULT_NB_RUNS = 1
    DEFAULT_STOP_AFTER_X_TIMEOUT = 10
    DEBUG = True

    def __init__(self, pathToInfrastructure: str, baseResult=None) -> None:
        """
        We initialize the class by reading the config file and getting the list of library and task.
        We also initialize the results dictionary and keep the path to the infrastructure

        Parameters
        ----------
        pathToInfrastructure : str
            path to the infrastructure

        Attributes
        ----------
        pathToInfrastructure : str
            path to the infrastructure
        results : dict
            dictionary that will contain the results and will format the json file
        libraryNames : list
            list of library name
        taskNames : list
            list of task name
        dictionaryTaskInTheme : dict of list of str
            dictionary that associate a theme to a list of task
        dictonaryThemeInTask : dict of str
            dictionary that associate a task to a theme
        """

        self.pathToInfrastructure = Path(pathToInfrastructure)

        # we collect the config into a dictionary
        self.libraryConfig = self.GetLibraryConfig()
        self.taskConfig = self.GetTaskConfig()
        self.themeConfig = self.GetThemeConfig()
        self.benchmark_config = self.GetBenchmarkConfig()

        logger.info(
            f"Library config retrieved: list of library {self.libraryConfig.keys()}"
        )
        logger.info(f"Task config retrieved: list of task {self.taskConfig.keys()}")
        logger.info(f"Theme config retrieved: list of theme {self.themeConfig.keys()}")

        themeDirectory = self.pathToInfrastructure / "themes"

        self.libraryNames = self.libraryConfig.keys()
        self.themeNames = [
            theme.name for theme in themeDirectory.iterdir() if theme.is_dir()
        ]

        self.taskNames = []
        self.dictionaryTaskInTheme = {}
        self.dictonaryThemeInTask = {}

        # create a dictionary that associate a theme to a list of task afnd a dictionary
        # that associate a task to a theme
        for themeName in self.themeNames:
            listTask = [
                task_path.name
                for task_path in themeDirectory.joinpath(themeName).iterdir()
                if task_path.is_dir()
            ]
            self.taskNames += listTask
            self.dictionaryTaskInTheme[themeName] = listTask
            for taskName in self.dictionaryTaskInTheme[themeName]:
                self.dictonaryThemeInTask[taskName] = themeName

        # We now rearange the order of execution of the tasks
        # We first look for the order in the config file
        # If no order is specified, we keep the order of the tasks in the config file
        order = []
        for themeName in self.themeNames:
            theme_config = self.themeConfig.get(themeName)

            if theme_config is None:
                continue

            order_in_theme = theme_config.get("task_order")
            if order_in_theme is None:
                continue
            order_in_theme = order_in_theme.split(",")
            for taskName in order_in_theme:
                order.append(
                    taskName.strip()
                ) if taskName.strip() in self.taskNames else None
        # we add the remaining tasks not yet added to the order
        for taskName in self.taskNames:
            if taskName not in order:
                order.append(taskName)

        self.taskNames = order

        # look for deactivated tasks
        deactivatedTasks = []
        for taskName in self.taskNames:
            if self.taskConfig[taskName].get("active") == "False":
                deactivatedTasks.append(taskName)

        # remove deactivated tasks from the list of tasks
        for taskName in deactivatedTasks:
            self.dictionaryTaskInTheme[self.dictonaryThemeInTask[taskName]].remove(
                taskName
            )
            self.taskNames.remove(taskName)

        logger.debug(f"active tasks: {self.taskNames}")

        if baseResult is None:
            self.results = self.create_base_json()
        else:
            self.results = self.get_result_from_json(baseResult)

        logger.debug(f"{self.dictionaryTaskInTheme = }")
        logger.debug(f"{self.dictonaryThemeInTask = }")

        # logger.debug(f"{self.results = }")
        logger.debug(f"{self.create_base_json() = }")

    def Setup_Global_Variables(self):
        Benchmark.DEBUG = eval(self.benchmark_config.get("debug", Benchmark.DEBUG))
        Benchmark.DEFAULT_TIMEOUT = int(
            self.benchmark_config.get("default_timeout", Benchmark.DEFAULT_TIMEOUT)
        )
        Benchmark.DEFAULT_NB_RUNS = int(
            self.benchmark_config.get("default_nb_runs", Benchmark.DEFAULT_NB_RUNS)
        )
        Benchmark.DEFAULT_STOP_AFTER_X_TIMEOUT = int(
            self.benchmark_config.get(
                "default_stop_after_x_timeout", Benchmark.DEFAULT_STOP_AFTER_X_TIMEOUT
            )
        )

    def get_result_from_json(self, json_file):
        """
        collect the results from a json file from a previous run
        """
        path_json = Path(json_file)
        if not path_json.exists():
            logger.error(f"File {json_file} does not exist")
            return self.create_base_json()

        # check suffix
        if path_json.suffix != ".json":
            logger.error(f"File {json_file} is not a json file")
            return self.create_base_json()

        with open(json_file, "r") as f:
            results = json.load(f)

        return results

    def create_base_json(self):
        """
        create the base json file structure if the json file does not exist

        """
        return {
            libraryName: {
                taskName: {
                    "theme": self.dictonaryThemeInTask[taskName],
                    "results": {
                        arg: {"runtime": []}
                        for arg in self.taskConfig[taskName].get("arguments").split(",")
                    },
                }
                for taskName in self.taskNames
            }
            for libraryName in self.libraryNames
        }

    def GetLibraryConfig(self):
        """
        get the library config from the config.ini file in the targets folder
        """
        strtest = StructureTest()
        libraryConfig = strtest.read_config(
            *strtest.find_config_file(self.pathToInfrastructure / "targets")
        )
        return libraryConfig

    def GetTaskConfig(self):
        """
        get the task config from the config.ini file in each task folder
        """
        strTest = StructureTest()
        listTaskpath = strTest.find_config_file(self.pathToInfrastructure / "themes")
        taskConfig = strTest.read_config(*listTaskpath)
        return taskConfig

    def GetThemeConfig(self):
        """
        get the theme config from the config.ini file in each theme folder
        """
        strTest = StructureTest()
        listThemepath = strTest.find_config_file(
            self.pathToInfrastructure / "themes", name="theme.ini"
        )
        themeConfig = strTest.read_config(*listThemepath)
        return themeConfig

    def GetBenchmarkConfig(self):
        """
        get the benchmark config from the config.ini file in the root folder
        """
        strTest = StructureTest()
        listBenchmarkpath = strTest.find_config_file(
            self.pathToInfrastructure / "config", name="config.ini"
        )
        benchmarkConfig = strTest.read_config(
            *listBenchmarkpath, listSection=["benchmark"]
        )
        return benchmarkConfig["config"]

    def BeforeBuildLibrary(self):
        """
        run the beforeBuild command of each library
        ussually used to build or update the library before running the task

        """
        logger.info(
            "Before build library ( we run the beforeBuild command of each library )"
        )
        for libraryName in self.libraryNames:
            process = subprocess.run(
                self.libraryConfig[libraryName].get("before_build"),
                shell=True,
                capture_output=True,
            )

            if process.returncode != 0:
                logger.error(f"Error in the beforeBuild command of {libraryName}")
                logger.debug(f"{process.stderr = }")
                raise Exception(
                    f"Error in the beforeBuild command of {libraryName} : {process.stderr}"
                )
            else:
                logger.info(f"Before build of {libraryName} done")

    def ExecuteFunctionInModule(self, module, funcName, **kwargs):
        """
        Execute a function in a module with the given arguments

        Parameters
        ----------
        module : str
            name of the module
        funcName : str
            name of the function
        kwargs : dict
            arguments of the function

        """
        logger.debug(f"Execute function {funcName} in module {module} with {kwargs}")
        res = None
        # first we check if the module exist
        if not os.path.exists(module.replace(".", os.sep) + ".py"):
            logger.warning(f"Module {module} does not exist")
            return Benchmark.NOT_RUN_VALUE
        try:
            module = __import__(module, fromlist=[funcName])
            func = getattr(module, funcName)
            res = func(**kwargs)
        except Exception as e:
            logger.warning(f"Error in the evaluation function {funcName}")
            logger.debug(f"{e = }")
            res = Benchmark.ERROR_VALUE
        finally:
            return res

    def BeforeTask(self, taskPath: str, taskName: str):
        """
        Run the before task command/script of a task if it exist

        Parameters
        ----------
        taskPath : str
            path to the task
        taskName : str
            name of the task

        """
        # check if the task has a before task command/script
        beforeTaskModule = self.taskConfig[taskName].get("before_script", None)
        # if the task has no before task command/script we do nothing
        if beforeTaskModule is None:
            logger.info(f"No before task command/script for {taskName}")
            return

        logger.info(f"Before task of {taskName}")

        # the before task is a function in a module
        funcName = self.taskConfig[taskName].get("before_function", None)
        logger.debug(f"{funcName = }")
        if funcName is None:
            logger.error(
                f"No function for the before task command/script for {taskName}"
            )
            return

        # the beforetask may have some arguments
        kwargs = self.taskConfig[taskName].get("before_task_arguments", "{}")
        kwargs = ast.literal_eval(kwargs)
        logger.debug(f"{kwargs = }")
        if len(kwargs) == 0:
            logger.warning(
                f"No arguments for the before task command/script for {taskName}"
            )

        # we import the module and run the function
        relativePath = os.path.relpath(
            taskPath, os.path.dirname(os.path.abspath(__file__))
        ).replace(os.sep, ".")

        
        # relative_module = __import__(f"{relativePath}.{beforeTaskModule}", fromlist=[funcName])
        # func = getattr(module, funcName)

        # logger.debug(f"{module.__name__ = }")
        # logger.debug(f"{func.__name__ = }")

        # try:
        #     func(**kwargs)
        # except Exception as e:
        #     logger.warning(f"Error in the evaluation function {funcName} of {taskName}")
        #     logger.debug(f"{e = }")

        # the before task should'nt return anything
        self.ExecuteFunctionInModule(
            f"{relativePath}.{beforeTaskModule}", funcName, **kwargs
        )

    def EvaluationAfterTask(
        self, moduleEvaluation, taskName: str, taskPath: str, *funcEvaluation, **kwargs
    ):
        """
        Run the evaluation function of a task

        Parameters
        ----------
        moduleEvaluation : str
            name of the module containing the evaluation function
        taskName : str
            name of the task
        taskPath : str
            path to the task
        funcEvaluation : str
            name of the evaluation function
        kwargs : dict
            arguments of the evaluation function

        """
        valueEvaluation = []

        if len(funcEvaluation) == 0:
            logger.warning(f"No evaluation function for {taskName}")
            return valueEvaluation

        # we need a relative path to import the module with os.sep replaced by .
        relativePath = os.path.relpath(
            taskPath, os.path.dirname(os.path.abspath(__file__))
        ).replace(os.sep, ".")
        # we remove the all first . in the relative path
        while relativePath[0] == ".":
            relativePath = relativePath[1:]

        for funcName in funcEvaluation:
            # command = f"{self.taskConfig[taskName].get('evaluation_language')} {os.path.join(taskPath,script)} {libraryName} {arg}"

            logger.debug(
                f"Run the evaluation function {funcName} of {moduleEvaluation} for {taskName} with {kwargs}"
            )

            # relative_module = __import__(
            #     f"{relativePath}.{moduleEvaluation}", fromlist=[funcName]
            # )
            # try:
            #     logger.debug(f"{module = }")
            #     func = getattr(module, funcName)
            #     logger.debug(f"{func = }")
            #     output = func(**kwargs)
            # except Exception as e:
            #     logger.warning(
            #         f"Error in the evaluation function {funcName} of {taskName}"
            #     )
            #     logger.debug(f"{e = }")
            #     output = Benchmark.ERROR_VALUE
            output = self.ExecuteFunctionInModule(
                f"{relativePath}.{moduleEvaluation}", funcName, **kwargs
            )
            logger.debug(f"{output = }")
            valueEvaluation.append(output)

        return valueEvaluation

    def RunProcess(self, command, timeout, getOutput=False):
        """
        Run a process with a timeout and return the time it took to run the process

        Parameters
        ----------
        command : str
            command to run
        timeout : int
            timeout in seconds
        getOutput : bool, optional
            if True return the output of the command, by default False
        """
        logger.debug(f"RunProcess with the command {command}")
        if Benchmark.DEBUG:
            return np.random.randint(low=5, high=10) * 1.0

        start = time.perf_counter()
        try:
            process = subprocess.run(
                command, shell=True, capture_output=True, text=True, timeout=timeout
            )
        except subprocess.TimeoutExpired:
            logger.warning(f"Timeout expired for the {command} command")
            return Benchmark.TIMEOUT_VALUE
        end = time.perf_counter()

        logger.debug(f"{process.stdout = }")
        logger.debug(f"{process.stderr = }")
        logger.debug(f"{process.returncode = }")

        if process.returncode == 1:
            logger.warning(f"Error in the command")
            return Benchmark.ERROR_VALUE

        elif process.returncode == 2:
            logger.warning(f"Can't run this command")
            return Benchmark.NOT_RUN_VALUE

        if getOutput:
            return process.stdout

        return end - start

    def CreateScriptName(self, libraryName: str, nameComplement="") -> str:
        """
        Create the name of the script that will be run for each library and task
        """
        suffix = {"python": "py", "java": "java", "c": "c", "c++": "cpp"}
        return f"{libraryName}{nameComplement}.{suffix[self.libraryConfig[libraryName].get('language', 'python')]}"

    def ScriptExist(self, scriptPath: str, scriptName: str) -> bool:
        """
        Check if the script exist in the path
        """
        script = Path(scriptPath) / scriptName
        return script.exists() and script.is_file()

    def RunTask(self, taskName: str):
        """
        Run the task for each library and save the results in the results dictionary
        """
        path = (
            self.pathToInfrastructure
            / "themes"
            / self.dictonaryThemeInTask[taskName]
            / taskName
        )

        #    We check if the before task command/script exist if not we do nothing
        beforeTaskModule = self.taskConfig[taskName].get("before_script", None)
        if beforeTaskModule is not None:
            self.BeforeTask(path, taskName)
        else:
            logger.info(f"No before task command/script for {taskName}")

        # The timeout of the task is the timeout in the config file or the default timeout
        # the timeout is in seconds
        taskTimeout = int(
            self.taskConfig[taskName].get("timeout", Benchmark.DEFAULT_TIMEOUT)
        )

        for libraryName in self.libraryNames:
            self.progressBar.set_description(
                f"Run task {taskName} for library {libraryName}"
            )

            self.RunTaskForLibrary(libraryName, taskName, path, timeout=taskTimeout)

    def TaskNotSupported(self, libraryName: str, taskName: str, arguments: str) -> None:
        # if task not supported by the target,
        # we add the results to the results dictionary with the value NOT_RUN_VALUE
        self.results[libraryName][taskName]["results"] = {
            arg: {"runtime": Benchmark.NOT_RUN_VALUE} for arg in arguments
        }
        # Progress bar update
        self.progressBar.update(
            int(self.taskConfig[taskName].get("nb_runs", Benchmark.DEFAULT_NB_RUNS))
            * len(arguments)
            * 2
        )  # *2 because we have before and after run script

    def RunTaskForLibrary(
        self, libraryName: str, taskName: str, taskPath: str, timeout: int
    ):
        arguments = self.taskConfig[taskName].get("arguments").split(",")

        # we check if the library support the task
        if not self.ScriptExist(taskPath, self.CreateScriptName(libraryName, "_run")):
            self.TaskNotSupported(libraryName, taskName, arguments)
            return

        logger.info(f"Run task {taskName} for library {libraryName}")

        # we check if there is a before run script
        before_run_script_exist = self.ScriptExist(
            taskPath, self.CreateScriptName(libraryName, "_before_run")
        )

        # we check if there is a after run script
        after_run_script = self.taskConfig[taskName].get("evaluation_script", None)

        stop_after_x_timeout = int(
            self.taskConfig[taskName].get(
                "stop_after_x_timeout", Benchmark.DEFAULT_STOP_AFTER_X_TIMEOUT
            )
        )
        stop_after_x_timeout = (
            float("inf") if stop_after_x_timeout <= 0 else stop_after_x_timeout
        )
        cpt_timeout = 0

        # runnning the task for each argument and the number of runs
        for arg in arguments:
            before_run_list_time = []
            listTime = []

            # number total of run for the task
            total_run = int(
                self.taskConfig[taskName].get("nb_runs", Benchmark.DEFAULT_NB_RUNS)
            )

            for cpt_run in range(total_run):
                logger.debug(
                    f"Run {cpt_run + 1} of {total_run} for {taskName}. At {cpt_timeout} timeout"
                )

                # Before run script
                if before_run_script_exist:
                    if cpt_timeout >= stop_after_x_timeout:
                        resultProcess = Benchmark.TIMEOUT_VALUE
                    else:
                        language = self.libraryConfig[libraryName].get("language")
                        path_script = Path(
                            taskPath, self.CreateScriptName(libraryName, "_before_run")
                        )
                        command = f"{language} {path_script} {arg}"

                        resultProcess = self.RunProcess(
                            command=command, timeout=timeout
                        )

                    before_run_list_time.append(resultProcess)
                    self.progressBar.update(1)
                    # if the before run script fail we don't run the task
                    # as the task is suposed to be an extension of the before run script
                    # if isinstance(resultProcess, str):
                    #     listTime.append(resultProcess)
                    # self.progressBar.update((total_run - cpt_run) * 2 - 1)
                    # self.progressBar.update(1)
                    # cpt_timeout += 1 if resultProcess == Benchmark.TIMEOUT_VALUE and cpt_run < stop_after_x_timeout else 0
                    # continue

                # Run script
                if cpt_timeout >= stop_after_x_timeout:
                    resultProcess = Benchmark.TIMEOUT_VALUE
                else:
                    scriptName = self.CreateScriptName(libraryName, "_run")
                    language = self.libraryConfig[libraryName].get("language")
                    path_script = Path(taskPath, scriptName)

                    command = f"{language} {path_script} {arg}"

                    resultProcess = self.RunProcess(
                        command=command,
                        timeout=timeout + resultProcess
                        if before_run_script_exist
                        and not isinstance(resultProcess, str)
                        else timeout,
                    )
                    logger.debug(f"{resultProcess = }")

                listTime.append(resultProcess)
                self.progressBar.update(1)
                # if the run script fail we just continue to the next run
                if isinstance(resultProcess, str):
                    #     # self.progressBar.update((total_run - nb_run - 1) * 2)
                    cpt_timeout += (
                        1
                        if resultProcess == Benchmark.TIMEOUT_VALUE
                        and cpt_run < stop_after_x_timeout
                        else 0
                    )
                #     continue

                # pass if in debug mode
                if Benchmark.DEBUG:
                    continue

                # After run script
                if after_run_script is not None:
                    # if the script is not None, then it should be a script name or a list of script name
                    functionEvaluation = self.taskConfig[taskName].get(
                        "evaluation_function", None
                    )
                    if functionEvaluation is not None:
                        functionEvaluation = functionEvaluation.split(" ")
                    else:
                        functionEvaluation = []

                    logger.debug(f"{functionEvaluation = }")

                    # if the task has  been run successfuly we run the evaluation function
                    if not isinstance(resultProcess, str):
                        valueEvaluation = self.EvaluationAfterTask(
                            after_run_script,
                            taskName,
                            taskPath,
                            *functionEvaluation,
                            libraryName=libraryName,
                            arg=arg,
                            **eval(
                                self.taskConfig[taskName].get(
                                    "evaluation_arguments", {}
                                )
                            ),
                        )
                        logger.debug(f"{valueEvaluation = }")
                    # if not we add the value ERROR_VALUE to the evaluation function
                    else:
                        valueEvaluation = [resultProcess] * len(functionEvaluation)
                    evaluation_result = self.results[libraryName][taskName]["results"][
                        arg
                    ].get("evaluation", {})
                    for i, function in enumerate(functionEvaluation):
                        element = evaluation_result.get(function, [])
                        evaluation_result = {
                            **evaluation_result,
                            function: element + [valueEvaluation[i]],
                        }
                    self.results[libraryName][taskName]["results"][arg][
                        "evaluation"
                    ] = evaluation_result

            self.results[libraryName][taskName]["results"][arg]["runtime"].extend(
                [b, t] for b, t in zip(before_run_list_time, listTime)
            )

        logger.info(f"End task {taskName} for library {libraryName}")

    def CalculNumberIteration(self):
        """
        Calculate the number of iteration for the progress bar
        """
        nbIteration = 0
        for taskName in self.taskNames:
            nbIteration += (
                int(self.taskConfig[taskName].get("nb_runs", Benchmark.DEFAULT_NB_RUNS))
                * len(self.taskConfig[taskName].get("arguments").split(","))
                * 2
                * len(self.libraryNames)
            )  # Nb runs * nb arguments * 2 (before run and after run) * nb libraries

        logger.info(f"Number of commands : {nbIteration}")
        return nbIteration

    def ConvertResultToJson(self, outputFileName="results.json"):
        """
        convert the result to a json file
        """
        with open(outputFileName, "w") as file:
            json.dump(self.results, file, indent=4)
        logger.info(f"Result saved in {outputFileName}")

    def StartAllProcedure(self):
        self.Setup_Global_Variables()
        if not Benchmark.DEBUG:
            self.BeforeBuildLibrary()

        self.progressBar = tqdm(
            total=self.CalculNumberIteration(),
            desc="Initialization",
            ncols=150,
            position=0,
        )
        logger.info("=======Begining of the benchmark=======")
        for taskName in self.taskNames:
            self.RunTask(taskName)
        logger.info("=======End of the benchmark=======")


if __name__ == "__main__":
    currentDirectory = Path(__file__).parent.absolute()
    outputPath = currentDirectory
    result_file = currentDirectory / "results.json"
    if result_file.exists():
        run = Benchmark(
            pathToInfrastructure=currentDirectory / "repository",
            baseResult=result_file.absolute(),
        )
    else:
        run = Benchmark(pathToInfrastructure=currentDirectory / "repository")

    # run = Benchmark(pathToInfrastructure=currentDirectory / "repository")
    run.StartAllProcedure()

    # print(run.results)
    run.ConvertResultToJson(result_file.absolute())
